# Generated by Django 4.1.10 on 2023-09-19 17:12

from django.db import migrations, models
import wagtail.blocks
import wagtail.fields


class Migration(migrations.Migration):

    dependencies = [
        ('geomanager', '0016_wmslayer_date_format'),
    ]

    operations = [
        migrations.AddField(
            model_name='wmslayer',
            name='more_info',
            field=wagtail.fields.StreamField([('more_info', wagtail.blocks.StructBlock([('link_text', wagtail.blocks.CharBlock(label='Link text', required=True)), ('link_url', wagtail.blocks.URLBlock(label='Link Url', required=True)), ('text', wagtail.blocks.TextBlock(label='Short description text', required=False)), ('is_button', wagtail.blocks.BooleanBlock(default=True, label='Format link as action button', required=False)), ('show_arrow', wagtail.blocks.BooleanBlock(default=True, label='Show arrow in button', required=False))], label='Info link'))], blank=True, null=True, use_json_field=True, verbose_name='More Info'),
        ),
        migrations.AlterField(
            model_name='wmslayer',
            name='date_format',
            field=models.CharField(blank=True, choices=[('yyyy-mm-dd HH:MM', 'Hourly - (E.g 2023-01-01 00:00)'), ('yyyy-mm-dd', 'Daily - (E.g 2023-01-01)'), ('yyyy-mm', 'Monthly - (E.g 2023-01)'), ('mmmm yyyy', 'Monthly - (E.g January 2023)'), ('pentadal', 'Pentadal - (E.g Jan 2023 - P1 1-5th)')], max_length=100, null=True),
        ),
    ]



import tempfile

from django.conf import settings
from django.db import connection, close_old_connections
from django.http import Http404, HttpResponse
from django.shortcuts import render, redirect
from django.urls import reverse
from django.utils.decorators import method_decorator
from django.utils.translation import gettext as _
from django.views import View
from wagtail.admin import messages
from wagtail.admin.auth import user_passes_test, user_has_any_page_permission
from wagtailcache.cache import clear_cache, cache_page

from geomanager.forms import StationsUploadForm, StationColumnsForm
from geomanager.models import StationSettings
from geomanager.utils.vector_utils import ogr_db_import


@user_passes_test(user_has_any_page_permission)
def load_stations(request):
    template = "geomanager/stations_upload.html"

    context = {}
    station_settings = StationSettings.for_request(request)

    if request.POST:
        form = StationsUploadForm(request.POST, request.FILES)

        if form.is_valid():
            shp_zip = form.cleaned_data.get("shp_zip")

            with tempfile.NamedTemporaryFile(delete=False, suffix=f"_{shp_zip.name}") as temp_file:
                for chunk in shp_zip.chunks():
                    temp_file.write(chunk)

                try:
                    default_db_settings = settings.DATABASES['default']
                    db_params = {
                        "host": default_db_settings.get("HOST"),
                        "port": default_db_settings.get("PORT"),
                        "user": default_db_settings.get("USER"),
                        "password": default_db_settings.get("PASSWORD"),
                        "name": default_db_settings.get("NAME"),
                    }

                    db_settings = {
                        **db_params,
                        "pg_service_schema": station_settings.db_schema
                    }

                    table_info = ogr_db_import(temp_file.name, station_settings.stations_table_name, db_settings,
                                               overwrite=True, validate_geom_types=["Point", "MultiPoint"])

                    columns = table_info.get("properties")
                    geom_type = table_info.get("geom_type")
                    bounds = table_info.get("bounds")

                    # set all columns to show in table and popup
                    for column in columns:
                        column["table"] = False
                        column["popup"] = True

                        # set label to be the name by default
                        column["label"] = column.get("name")

                    # update stations settings
                    station_settings.columns = columns
                    station_settings.geom_type = geom_type
                    station_settings.bounds = bounds
                    station_settings.name_column = ""
                    station_settings.save()

                except Exception as e:
                    form.add_error(None, str(e))
                    context.update({"form": form})
                    return render(request, template_name=template, context=context)

            messages.success(request, _("Stations data loaded successfully"))

            # clear wagtail cache
            clear_cache()

            return redirect(reverse("geomanager_preview_stations"))
        else:
            context.update({"form": form})
            return render(request, template_name=template, context=context)
    else:
        form = StationsUploadForm()
        context["form"] = form
        return render(request, template_name=template, context=context)


@user_passes_test(user_has_any_page_permission)
def preview_stations(request):
    template = "geomanager/stations_preview.html"

    stations_settings = StationSettings.for_request(request)
    stations_vector_tiles_url = request.scheme + '://' + request.get_host() + \
                                stations_settings.stations_vector_tiles_url

    context = {
        "mapConfig": {
            "stationBounds": stations_settings.bounds,
            "stationsVectorTilesUrl": stations_vector_tiles_url,
        },
        "load_stations_url": reverse("geomanager_load_stations"),
    }

    if stations_settings.columns:
        context.update({"station_columns": stations_settings.columns})

    initial_data = {
        "columns": stations_settings.columns,
        "name_column": stations_settings.name_column
    }

    column_choices = [(column, column) for column in stations_settings.station_columns_list]

    if request.POST:
        form = StationColumnsForm(request.POST, initial=initial_data, column_choices=column_choices)
        if form.is_valid():
            columns = form.cleaned_data.get("columns")
            name_column = form.cleaned_data.get("name_column")

            if columns:
                stations_settings.columns = columns
                stations_settings.name_column = name_column
                stations_settings.save()
                # clear wagtail cache
                clear_cache()
            messages.success(request, _("Stations columns updated successfully"))

            # redirect
            return redirect(reverse("geomanager_preview_stations"))
        else:
            context.update({"form": form})
            return render(request, template_name=template, context=context)
    else:
        form = StationColumnsForm(initial=initial_data, column_choices=column_choices)
        context["form"] = form

    return render(request, template, context=context)


@method_decorator(cache_page, name='get')
class StationsTileView(View):
    def get(self, request, z, x, y):
        station_settings = StationSettings.for_request(request)

        sql = f"""WITH
            bounds AS (
              SELECT ST_TileEnvelope({z}, {x}, {y}) AS geom
            ),
            mvtgeom AS (
              SELECT ST_AsMVTGeom(ST_Transform(t.geom, 3857), bounds.geom) AS geom,
                *
              FROM {station_settings.full_table_name} t, bounds
              WHERE ST_Intersects(ST_Transform(t.geom, 4326), ST_Transform(bounds.geom, 4326))
            )
            SELECT ST_AsMVT(mvtgeom, 'default') FROM mvtgeom;
            """

        close_old_connections()
        with connection.cursor() as cursor:
            cursor.execute(sql)
            tile = cursor.fetchone()[0]
            if not len(tile):
                raise Http404()

        return HttpResponse(tile, content_type="application/x-protobuf")



from adminboundarymanager.models import AdminBoundarySettings
from django.utils.decorators import method_decorator
from rest_framework import mixins, viewsets
from rest_framework.renderers import JSONRenderer
from rest_framework.response import Response
from wagtailcache.cache import cache_page

from geomanager import serializers
from geomanager.models import Dataset
from geomanager.models.core import GeomanagerSettings, Metadata


class DatasetViewSet(mixins.ListModelMixin, mixins.RetrieveModelMixin, viewsets.GenericViewSet):
    queryset = Dataset.objects.filter(published=True)
    serializer_class = serializers.DatasetSerializer

    renderer_classes = [JSONRenderer]

    def get_serializer_context(self):
        context = super().get_serializer_context()
        context.update({'request': self.request})
        return context

    @method_decorator(cache_page)
    def list(self, request, *args, **kwargs):
        queryset = self.get_queryset()
        dataset_with_layers = []

        # get only datasets with layers defined
        for dataset in queryset:
            if dataset.has_layers():
                dataset_with_layers.append(dataset)

        serializer = self.get_serializer(dataset_with_layers, many=True)

        lm_settings = GeomanagerSettings.for_request(request)
        abm_settings = AdminBoundarySettings.for_request(request)
        datasets = serializer.data
        config = {}

        # set boundaries url. This will be used to create base boundary layer
        boundary_tiles_url = abm_settings.boundary_tiles_url
        boundary_tiles_url = request.scheme + '://' + request.get_host() + boundary_tiles_url
        config.update({"boundaryTilesUrl": boundary_tiles_url})

        # set cap layer configuration. This will be used to create cap layer
        if lm_settings.cap_base_url and lm_settings.cap_sub_category:
            config.update({
                "capConfig": {
                    "initialVisible": lm_settings.cap_shown_by_default,
                    "baseUrl": lm_settings.cap_base_url,
                    "category": lm_settings.cap_sub_category.category.pk,
                    "subCategory": lm_settings.cap_sub_category.pk,
                    "refreshInterval": lm_settings.cap_auto_refresh_interval_milliseconds,
                    "metadata": lm_settings.cap_metadata.pk if lm_settings.cap_metadata else None
                }})

        return Response({"datasets": datasets, "config": config})


@method_decorator(cache_page, name="retrieve")
class MetadataViewSet(mixins.RetrieveModelMixin, viewsets.GenericViewSet):
    queryset = Metadata.objects.all()
    serializer_class = serializers.MetadataSerialiazer



from adminboundarymanager.models import AdminBoundarySettings
from rest_framework.decorators import api_view, renderer_classes
from rest_framework.renderers import JSONRenderer
from rest_framework.response import Response
from wagtailcache.cache import cache_page

from geomanager.models import Category, VectorLayerIcon, VectorTileLayerIcon
from geomanager.models.core import GeomanagerSettings
from geomanager.serializers import CategorySerializer


@api_view(['GET'])
@cache_page
@renderer_classes([JSONRenderer])
def get_mapviewer_config(request):
    gm_settings = GeomanagerSettings.for_request(request)
    abm_settings = AdminBoundarySettings.for_request(request)

    categories = Category.objects.all()
    categories_data = CategorySerializer(categories, many=True).data
    response = {
        "categories": categories_data,
    }

    links = {
        "mapViewerBaseUrl": request.build_absolute_uri("/mapviewer"),
    }

    if gm_settings.terms_of_service_page:
        links.update({"termsOfServicePageUrl": request.build_absolute_uri(gm_settings.terms_of_service_page.url)})

    if gm_settings.privacy_policy_page:
        links.update({"privacyPolicyPageUrl": request.build_absolute_uri(gm_settings.privacy_policy_page.url)})

    response.update({"links": links})

    icon_images = []
    for icon in VectorLayerIcon.objects.all():
        icon_images.append({"name": icon.name, "url": request.build_absolute_uri(icon.file.url)})

    for icon in VectorTileLayerIcon.objects.all():
        icon_images.append({"name": icon.name, "url": request.build_absolute_uri(icon.file.url)})

    response.update({"vectorLayerIcons": icon_images})

    if gm_settings.logo:
        logo = {
            "imageUrl": request.build_absolute_uri(gm_settings.logo.file.url)
        }

        if gm_settings.logo_page:
            logo.update({"linkUrl": request.build_absolute_uri(gm_settings.logo_page.url)})

        if not gm_settings.logo_page and gm_settings.logo_external_link:
            logo.update({"linkUrl": gm_settings.logo_external_link, "external": True})

        response.update({"logo": logo})

    if abm_settings.countries_list:
        response.update({
            "countries": abm_settings.countries_list,
            "bounds": abm_settings.combined_countries_bounds,
            "boundaryDataSource": abm_settings.data_source
        })

    base_maps_data = []

    tile_gl_source = gm_settings.tile_gl_source

    if tile_gl_source:
        # get base maps
        for base_map in gm_settings.base_maps:
            data = base_map.block.get_api_representation(base_map.value)
            for key, value in base_map.value.items():
                if key == "image" and value:
                    data.update({"image": request.build_absolute_uri(value.file.url)})

            data.update({"mapStyle": request.build_absolute_uri(tile_gl_source.map_style_url)})
            base_maps_data.append(data)

    response.update({"basemaps": base_maps_data})

    return Response(response)



# Generated by Django 4.2.1 on 2023-06-08 08:23

from django.db import migrations, models
import django.db.models.deletion
import wagtail.blocks
import wagtail.fields


class Migration(migrations.Migration):

    dependencies = [
        ('wagtailimages', '0025_alter_image_file_alter_rendition_file'),
        ('geomanager', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='geomanagersettings',
            name='logo',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='wagtailimages.image', verbose_name='Logo'),
        ),
        migrations.AddField(
            model_name='geomanagersettings',
            name='navigation',
            field=wagtail.fields.StreamField([('menu_items', wagtail.blocks.ListBlock(wagtail.blocks.StructBlock([('label', wagtail.blocks.CharBlock(label='Label')), ('page', wagtail.blocks.PageChooserBlock(label='Page'))])))], blank=True, null=True, use_json_field=True),
        ),
    ]



from django.contrib.gis.db import models
from django.urls import reverse
from django.utils.functional import cached_property
from django.utils.html import format_html
from django_tables2 import tables, LazyPaginator, TemplateColumn
from wagtail.contrib.routable_page.models import RoutablePageMixin, path
from wagtail.contrib.settings.models import BaseSiteSetting
from wagtail.models import Page

from geomanager.fields import ListField
from geomanager.utils.vector_utils import get_model_field


class StationSettings(BaseSiteSetting):
    stations_table_name = "geomanager_station"
    db_schema = "public"

    columns = models.JSONField(blank=True, null=True)
    geom_type = models.CharField(max_length=100, blank=True, null=True)
    bounds = ListField(max_length=256, blank=True, null=True)
    name_column = models.CharField(max_length=100, blank=True, null=True)

    @cached_property
    def full_table_name(self):
        return f"{self.db_schema}.{self.stations_table_name}"

    @cached_property
    def stations_vector_tiles_url(self):
        base_url = reverse("station_tiles", args=(0, 0, 0)).replace("/0/0/0", r"/{z}/{x}/{y}")
        return base_url

    def get_station_model(self):
        fields = self.station_fields_factory()

        attrs = {
            **fields,
            "managed": False,
            "__module__": "geomanager.station"
        }

        station_model = type("Station", (models.Model,), attrs)

        return station_model

    def station_fields_factory(self):
        fields = {
            "geom": get_model_field(self.geom_type)()
        }

        if isinstance(self.columns, list):
            for column in self.columns:
                data_type = column.get("data_type")
                name = column.get("name")
                label = column.get("label") or name
                if data_type:
                    model_field = get_model_field(column.get("data_type"))

                    if model_field:
                        field_kwargs = {"verbose_name": label}
                        if name == "gid":
                            field_kwargs.update({"primary_key": True})
                        fields.update({name: model_field(**field_kwargs)})

        return fields

    @cached_property
    def station_columns_list(self):
        station_columns = []
        if self.columns and isinstance(self.columns, list):
            for column in self.columns:
                name = column.get("name")
                if name:
                    station_columns.append(name)
        return station_columns

    @cached_property
    def station_table_columns_list(self):
        table_columns = []
        if self.columns and isinstance(self.columns, list):
            for column in self.columns:
                name = column.get("name")
                label = column.get("label")
                table = column.get("table")
                if name and table:
                    table_columns.append({"name": name, "label": label})
        return table_columns

    @cached_property
    def station_popup_columns_list(self):
        popup_columns = []
        if self.columns and isinstance(self.columns, list):
            for column in self.columns:
                name = column.get("name")
                label = column.get("label")
                popup = column.get("popup")
                if name and popup:
                    popup_columns.append({"name": name, "label": label})
        return popup_columns


class AbstractStationsPage(RoutablePageMixin, Page):
    class Meta:
        abstract = True

    @path('')
    def all_stations(self, request, *args, **kwargs):
        context = {}
        station_settings = StationSettings.for_request(request)

        stations_vector_tiles_url = request.scheme + '://' + request.get_host() + station_settings.stations_vector_tiles_url

        context.update({
            "mapConfig": {
                "stationBounds": station_settings.bounds,
                "stationsVectorTilesUrl": stations_vector_tiles_url,
            },
        })

        # get stations model
        station_model = station_settings.get_station_model()

        # get all columns
        station_table_columns_list = station_settings.station_table_columns_list

        table_fields = [field.get("name") for field in station_table_columns_list]

        page_url = request.build_absolute_uri(self.url)

        class StationTable(tables.Table):
            detail_url = TemplateColumn('<a href="" target="_blank"></a>')

            class Meta:
                model = station_model
                fields = table_fields

            def render_detail_url(self, value, record):
                record_pk_suffix = str(record.gid)
                if page_url and not page_url.endswith("/"):
                    record_pk_suffix = f"/{record_pk_suffix}"
                url = page_url + record_pk_suffix
                return format_html(
                    "<a href='{}'>View detail</a>",
                    url
                )

        stations_table = StationTable(station_model.objects.all())
        stations_table.paginate(page=request.GET.get("page", 1), per_page=50, paginator_class=LazyPaginator)

        context.update({
            "stations_table": stations_table,
            "popup_fields": station_settings.station_popup_columns_list
        })

        return self.render(request, context_overrides={**context})

    @path('<int:station_pk>/')
    def station_detail(self, request, station_pk):
        station_settings = StationSettings.for_request(request)

        # get stations model
        station_model = station_settings.get_station_model()

        station = station_model.objects.filter(pk=station_pk)

        if station.exists:
            station = station.first()
        else:
            station = None

        context = {
            "station": station,
            "columns": station_settings.columns,
            "bounds": station_settings.bounds,
            "station_name_column": station_settings.name_column
        }

        return self.render(request, template="stations/station_detail_page.html", context_overrides=context)



import uuid

from django.db import models
from django.utils.translation import gettext_lazy as _
from django_extensions.db.models import TimeStampedModel
from modelcluster.fields import ParentalKey
from modelcluster.models import ClusterableModel
from wagtail import blocks
from wagtail.admin.panels import (
    FieldPanel,
    TabbedInterface,
    ObjectList,
    InlinePanel
)
from wagtail_modeladmin.helpers import AdminURLHelper
from wagtail.contrib.settings.models import BaseSiteSetting
from wagtail.contrib.settings.registry import register_setting
from wagtail.fields import StreamField, RichTextField
from wagtail.images.blocks import ImageChooserBlock
from wagtail.models import Orderable, Page
from wagtail_adminsortable.models import AdminSortable
from wagtailiconchooser.widgets import IconChooserWidget

from geomanager.helpers import (
    get_layer_action_url,
    get_preview_url,
    get_upload_url
)
from .tile_gl import MBTSource
from ..blocks import NavigationItemsBlock

DEFAULT_RASTER_MAX_UPLOAD_SIZE_MB = 10


class Category(TimeStampedModel, AdminSortable, ClusterableModel):
    title = models.CharField(max_length=16, verbose_name=_("title"), help_text=_("Title of the category"))
    icon = models.CharField(max_length=255, verbose_name=_("icon"), blank=True, null=True)
    active = models.BooleanField(default=True, verbose_name=_("active"), help_text=_("Is the category active ?"))
    public = models.BooleanField(default=True, verbose_name=_("public"), help_text=_("Is the category public ?"))

    class Meta(AdminSortable.Meta):
        verbose_name = _("Category")
        verbose_name_plural = _("Categories")

    def __str__(self):
        return self.title

    panels = [
        FieldPanel("title"),
        FieldPanel("icon", widget=IconChooserWidget),
        FieldPanel("active"),
        FieldPanel("public"),

        InlinePanel("sub_categories", heading=_("Sub Categories"), label=_("Sub Category")),
    ]

    def datasets_list_url(self):
        dataset_admin_helper = AdminURLHelper(Dataset)
        dataset_index_url = dataset_admin_helper.get_action_url("index")
        return dataset_index_url + f"?category__id__exact={self.pk}"

    def dataset_create_url(self):
        dataset_admin_helper = AdminURLHelper(Dataset)
        dataset_create_url = dataset_admin_helper.get_action_url("create")
        return dataset_create_url + f"?category_id={self.pk}"


class SubCategory(Orderable):
    category = ParentalKey(Category, on_delete=models.CASCADE, related_name="sub_categories")
    title = models.CharField(max_length=256, verbose_name=_("title"))
    active = models.BooleanField(default=True, verbose_name=_("active"))
    public = models.BooleanField(default=True, verbose_name=_("public"))

    class Meta:
        verbose_name = _("Subcategory")
        verbose_name_plural = _("Subcategories")

    panels = [
        FieldPanel("title"),
        FieldPanel("active"),
        FieldPanel("public"),
    ]

    def __str__(self):
        return self.title


class Dataset(TimeStampedModel):
    DATASET_TYPE_CHOICES = (
        ("raster_file", _("Raster File - NetCDF/GeoTiff")),
        ("vector_file", _("Vector File - Shapefile, Geojson")),
        ("wms", _("Web Map Service - WMS Layer")),
        ("raster_tile", _("XYZ Raster Tile Layer")),
        ("vector_tile", _("XYZ Vector Tile Layer")),
    )

    CURRENT_TIME_METHOD_CHOICES = (
        ("latest_from_source", _("Latest From Source")),
        ("previous_to_now", _("Previous to now")),
        ("next_to_now", _("Next to Now")),
    )

    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    title = models.CharField(max_length=255, verbose_name=_("title"),
                             help_text=_("The Dataset title as will appear to the public"))
    category = models.ForeignKey(Category, verbose_name=_("category"), on_delete=models.PROTECT)
    sub_category = models.ForeignKey(SubCategory, verbose_name=_("Subcategory"), on_delete=models.PROTECT)
    summary = models.CharField(max_length=100, null=True, blank=True,
                               verbose_name=_("summary"),
                               help_text=_("Short summary of less than 100 characters"))
    metadata = models.ForeignKey("Metadata", verbose_name=_("metadata"), null=True, blank=True,
                                 on_delete=models.SET_NULL)
    layer_type = models.CharField(max_length=100, choices=DATASET_TYPE_CHOICES, default="raster_file",
                                  verbose_name=_("Layer type"))
    published = models.BooleanField(default=True, verbose_name=_("published"),
                                    help_text=_("Should the dataset be available for visualization ?"
                                                " If unchecked, the dataset is assumed to be in draft mode "
                                                "and thus not ready"))
    public = models.BooleanField(default=True, verbose_name=_("public"),
                                 help_text=_("Should the dataset be visible to everyone ?"
                                             " If unchecked, only authorized users can view"))
    initial_visible = models.BooleanField(default=False, verbose_name=_("Initially visible on Map by default"),
                                          help_text=_("Make the dataset visible on the map by default"))

    multi_temporal = models.BooleanField(default=True, verbose_name=_("Multi-temporal"),
                                         help_text=_("The dataset is multi-temporal"), )

    multi_layer = models.BooleanField(default=False, verbose_name=_("Multi-layer"),
                                      help_text=_("The dataset has more than one layer, to be displayed together"), )

    near_realtime = models.BooleanField(default=False, verbose_name=_("Near realtime"),
                                        help_text=_(
                                            "Is the layer near realtime?, for example updates every 10 minutes"))

    current_time_method = models.CharField(max_length=100, choices=CURRENT_TIME_METHOD_CHOICES,
                                           default="latest_from_source",
                                           verbose_name=_("current time method"),
                                           help_text=_(
                                               "How to pick default time and for updates, for Multi-Temporal data"))
    auto_update_interval = models.IntegerField(blank=True, null=True, verbose_name=_("Auto Update interval in minutes"),
                                               help_text=_(
                                                   "After how many minutes should the layer auto update on the map to "
                                                   "show current data, if multi-temporal. Leave empty to"
                                                   " disable auto updating"))

    can_clip = models.BooleanField(default=True, verbose_name=_("Enable Clipping by shape"),
                                   help_text=_("Check to enable clipping by boundary or drawn shapes, "
                                               "for raster and vector datasets. Not implemented for WMS types"))

    class Meta:
        verbose_name = _("Dataset")
        verbose_name_plural = _("Datasets")

    panels = [
        FieldPanel("title"),
        FieldPanel("category"),
        FieldPanel("sub_category"),
        FieldPanel("layer_type"),
        FieldPanel("summary"),
        FieldPanel("metadata"),
        FieldPanel("published"),
        FieldPanel("public"),
        FieldPanel("initial_visible"),
        FieldPanel("multi_temporal"),
        FieldPanel("multi_layer"),
        FieldPanel("near_realtime"),
        FieldPanel("current_time_method"),
        FieldPanel("auto_update_interval"),
        FieldPanel("can_clip"),
    ]

    def __str__(self):
        return self.title

    @property
    def auto_update_interval_milliseconds(self):
        if self.auto_update_interval:
            return self.auto_update_interval * 60000
        return None

    @property
    def capabilities(self):
        caps = []
        if self.multi_temporal:
            caps.append("timeseries")
        if self.near_realtime:
            caps.append("nearRealTime")
        return caps

    def dataset_url(self):
        admin_helper = AdminURLHelper(self)
        admin_edit_url = admin_helper.get_action_url("index", self.pk)
        return admin_edit_url + f"?id={self.pk}"

    def get_layers_rel(self):
        layer_type = self.layer_type

        if layer_type == "raster_file":
            return self.raster_file_layers

        if layer_type == "vector_file":
            return self.vector_file_layers

        if layer_type == "wms":
            return self.wms_layers

        if layer_type == "raster_tile":
            return self.raster_tile_layers

        if layer_type == "vector_tile":
            return self.vector_tile_layers

        return None

    @property
    def category_url(self):
        if self.category:
            category_admin_helper = AdminURLHelper(Category)
            category_edit_url = category_admin_helper.get_action_url("edit", self.category.pk)
            return category_edit_url
        return None

    @property
    def upload_url(self):
        return get_upload_url(self.layer_type, self.pk)

    def layers_list_url(self):
        list_layer_url = get_layer_action_url(self.layer_type, "index")

        if list_layer_url:
            list_layer_url = list_layer_url + f"?dataset__id__exact={self.pk}"

        return list_layer_url

    def create_layer_url(self):
        if self.has_layers() and not self.multi_layer:
            return None

        create_layer_url = get_layer_action_url(self.layer_type, "create")
        if create_layer_url:
            create_layer_url = create_layer_url + f"?dataset_id={self.pk}"

        return create_layer_url

    @property
    def preview_url(self):
        return get_preview_url(self.layer_type, self.pk)

    def has_layers(self):
        layers = self.get_layers_rel()
        if layers:
            return layers.exists()

        return False

    def can_preview(self):
        layers_check = [
            self.has_raster_files(),
            self.has_vector_tables(),
            self.has_wms_layers(),
            self.has_raster_tile_layers(),
            self.has_vector_tile_layers()
        ]

        return any(layers_check)

    def has_raster_files(self):
        layers = self.raster_file_layers.all()
        has_raster_files = False
        if layers.exists():
            for layer in layers:
                if layer.raster_files.exists():
                    has_raster_files = True
                    break
        return has_raster_files

    def has_vector_tables(self):
        vector_layers = self.vector_file_layers.all()
        has_vector_tables = False
        if vector_layers.exists():
            for layer in vector_layers:
                if layer.vector_tables.all().exists():
                    has_vector_tables = True
                    break
        return has_vector_tables

    def has_wms_layers(self):
        return self.wms_layers.exists()

    def has_raster_tile_layers(self):
        return self.raster_tile_layers.exists()

    def has_vector_tile_layers(self):
        return self.vector_tile_layers.exists()

    def get_default_layer(self):
        layers = self.get_layers_rel()

        if layers and layers.exists():
            default = layers.filter(default=True)
            if default.exists():
                return default.first().pk
            else:
                return layers.first().pk

        return None

    def get_wms_layers_json(self):
        return []


class Metadata(TimeStampedModel):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    title = models.CharField(max_length=255, verbose_name=_("title"), help_text=_("Title of the dataset"))
    subtitle = models.CharField(max_length=255, blank=True, null=True, verbose_name=_("subtitle"),
                                help_text=_("Subtitle if any"))
    function = models.TextField(max_length=255, blank=True, null=True, verbose_name=_("Dataset summary"),
                                help_text=_("Short summary of what the dataset shows. Keep it short."))
    resolution = models.CharField(max_length=255, blank=True, null=True, verbose_name=_("resolution"),
                                  help_text=_("The Spatial resolution of the dataset, for example 10km by 10 km"))
    geographic_coverage = models.CharField(max_length=255, blank=True, null=True, verbose_name=_("Geographic coverage"),
                                           help_text=_("The geographic coverage of the dataset. For example East Africa"
                                                       " or specific country name like Ethiopia, or Africa"))
    source = models.CharField(max_length=255, blank=True, null=True, verbose_name=_("source"),
                              help_text=_("The source of the data where was it generated or produced"))
    license = models.CharField(max_length=255, blank=True, null=True, verbose_name=_("license"),
                               help_text=_("Any licensing information for the dataset"))
    frequency_of_update = models.CharField(max_length=255, blank=True, null=True,
                                           verbose_name=_("Frequency of updates"),
                                           help_text=_("How frequent is the dataset updated. "
                                                       "For example daily, weekly, monthly etc"))
    overview = RichTextField(blank=True, null=True, verbose_name=_("detail"),
                             help_text=_("Detail description of the dataset, including the methodology, "
                                         "references or any other relevant information"))
    cautions = RichTextField(blank=True, null=True, verbose_name=_("cautions"),
                             help_text=_("What things should users be aware as they use and interpret this dataset"))
    citation = RichTextField(blank=True, null=True, verbose_name=_("citation"),
                             help_text=_("Scientific citation for this dataset if any. "
                                         "For example the citation for a scientific paper for the dataset"))
    download_data = models.URLField(blank=True, null=True, verbose_name=_("Data download link"),
                                    help_text=_("External link to where the source data can be found and downloaded"))
    learn_more = models.URLField(blank=True, null=True, verbose_name=_("Learn more link"),
                                 help_text=_("External link to where more detail about the dataset can be found"))

    class Meta:
        verbose_name = _("Metadata")
        verbose_name_plural = _("Metadata")

    def __str__(self):
        return self.title


def get_styles():
    from geomanager.models import MBTSource
    return [(style.id, style.name) for style in MBTSource.objects.all()]


class BaseLayer(models.Model):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    title = models.CharField(max_length=255, verbose_name=_("title"), help_text=_("Layer title"))
    default = models.BooleanField(default=False, verbose_name=_("default"), help_text=_("Is Default Layer"))

    @property
    def edit_url(self):
        edit_url = get_layer_action_url(layer_type=self.dataset.layer_type, action="edit", action_args=self.pk)
        return edit_url

    @property
    def upload_url(self):
        return get_upload_url(layer_type=self.dataset.layer_type, dataset_id=self.dataset.pk, layer_id=self.pk)

    @property
    def preview_url(self):
        return get_preview_url(layer_type=self.dataset.layer_type, dataset_id=self.dataset.pk, layer_id=self.pk)

    class Meta:
        abstract = True


@register_setting
class GeomanagerSettings(BaseSiteSetting):
    GADM_VERSION = (
        ("4.1", "4.1"),
    )

    max_upload_size_mb = models.IntegerField(default=DEFAULT_RASTER_MAX_UPLOAD_SIZE_MB,
                                             verbose_name=_("Maximum upload size in MegaBytes"),
                                             help_text=_(
                                                 "Maximum raster file size that can be uploaded in MegaBytes. "
                                                 "Default is 10Mbs."))
    crop_raster_to_country = models.BooleanField(default=True, verbose_name=_("Crop raster to country"),
                                                 help_text=_("Crop the uploaded raster file to the country boundaries"))

    cap_base_url = models.URLField(max_length=256, null=True, blank=True, verbose_name=_("cap base url"))
    cap_sub_category = models.ForeignKey(SubCategory, null=True, blank=True, verbose_name=_("cap layer sub category"),
                                         on_delete=models.SET_NULL)
    cap_auto_refresh_interval = models.IntegerField(blank=True, null=True,
                                                    verbose_name=_("Auto Refresh interval in minutes"),
                                                    help_text=_(
                                                        "Refresh cap alerts on the map after this minutes. Leave blank "
                                                        "to disable auto refreshing"))
    cap_shown_by_default = models.BooleanField(default=True, verbose_name=_("CAP layer shown by default"),
                                               help_text=_("CAP Layer shown on map by default"))
    cap_metadata = models.ForeignKey(Metadata, on_delete=models.SET_NULL, blank=True, null=True,
                                     verbose_name=_("Metadata"))
    gadm_version = models.CharField(max_length=50, choices=GADM_VERSION, default="4.1",
                                    verbose_name=_("GADM Boundary Data Version"),
                                    help_text=_(
                                        "It is encouraged to use the latest version. "
                                        "See available versions here: https://gadm.org/old_versions.html"))
    tile_gl_fonts_url = models.URLField(max_length=256,
                                        default="https://fonts.openmaptiles.org/{fontstack}/{range}.pbf",
                                        verbose_name=_("GL Styles Font Url"),
                                        help_text=_("GL Styles Font Url"))
    tile_gl_source = models.ForeignKey(MBTSource, blank=True, null=True, on_delete=models.SET_NULL,
                                       verbose_name=_("Open Map Tiles Source"))
    logo = models.ForeignKey(
        'wagtailimages.Image',
        null=True,
        blank=True,
        on_delete=models.SET_NULL,
        related_name='+',
        verbose_name=_("Logo")
    )
    logo_page = models.ForeignKey(
        Page,
        null=True,
        blank=True,
        on_delete=models.SET_NULL,
        related_name='+',
        verbose_name=_("Logo page"),
        help_text=_("Internal page to navigate to on clicking the logo")
    )
    logo_external_link = models.URLField(max_length=255, null=True, blank=True,
                                         verbose_name=_("Logo external link"),
                                         help_text=_("Used if internal logo page not provided"))
    terms_of_service_page = models.ForeignKey(
        Page,
        null=True,
        blank=True,
        on_delete=models.SET_NULL,
        related_name='+',
        verbose_name=_("Terms of Service Page"),
        help_text=_("MapViewer Terms of Service page")
    )
    privacy_policy_page = models.ForeignKey(
        Page,
        null=True,
        blank=True,
        on_delete=models.SET_NULL,
        related_name='+',
        verbose_name=_("Privacy Policy Page"),
        help_text=_("MapViewer Privacy Policy Page")
    )

    navigation = StreamField([
        ('menu_items', blocks.ListBlock(NavigationItemsBlock(max_num=8))),
    ], block_counts={
        'menu_items': {'max_num': 1},
    }, use_json_field=True, null=True, blank=True)

    base_maps = StreamField([
        ('basemap', blocks.StructBlock([
            ('label', blocks.CharBlock(label=_("label"))),
            ('backgroundColor', blocks.CharBlock(label=_("background color"))),
            ('image', ImageChooserBlock(required=False, label=_("image"))),
            ('basemapGroup', blocks.CharBlock(label=_("basemap group"))),
            ('labelsGroup', blocks.CharBlock(label=_("labels group"))),
            ('url', blocks.URLBlock(required=False, label=_("url"))),
            ('default', blocks.BooleanBlock(required=False, label=_("default"), help_text=_("Is default style ?"))),
        ]))
    ], use_json_field=True, null=True, blank=True)

    class Meta:
        verbose_name = _("Geomanager Settings")

    edit_handler = TabbedInterface([
        ObjectList([
            FieldPanel("max_upload_size_mb"),
            FieldPanel("crop_raster_to_country"),
        ], heading=_("Upload Settings")),
        ObjectList([
            FieldPanel("tile_gl_fonts_url"),
            FieldPanel("tile_gl_source"),
            FieldPanel("base_maps"),
        ], heading=_("Basemap TileServer Settings")),
        ObjectList([
            FieldPanel("cap_base_url"),
            FieldPanel("cap_sub_category"),
            FieldPanel("cap_auto_refresh_interval"),
            FieldPanel("cap_shown_by_default"),
            FieldPanel("cap_metadata"),
        ], heading=_("CAP Layer Settings")),
        ObjectList([
            FieldPanel("logo"),
            FieldPanel("logo_page"),
            FieldPanel("logo_external_link"),
            FieldPanel("terms_of_service_page"),
            FieldPanel("privacy_policy_page"),
            FieldPanel("navigation"),
        ], heading=_("Navigation Settings")),
    ])

    @property
    def cap_auto_refresh_interval_milliseconds(self):
        if self.cap_auto_refresh_interval:
            return self.cap_auto_refresh_interval * 60000
        return None

    @property
    def max_upload_size_bytes(self):
        return self.max_upload_size_mb * 1024 * 1024



from django.contrib.admin.utils import quote
from django.core.files.base import ContentFile
from django.db import models
from django.urls import reverse
from django.utils.translation import gettext_lazy as _
from modelcluster.fields import ParentalKey
from wagtail import blocks
from wagtail.admin.panels import FieldPanel
from wagtail.fields import StreamField

from geomanager.blocks import (
    FillVectorLayerBlock,
    LineVectorLayerBlock,
    CircleVectorLayerBlock,
    IconVectorLayerBlock,
    TextVectorLayerBlock,
)
from geomanager.models.core import Dataset
from geomanager.models.tile_base import BaseTileLayer
from geomanager.utils.svg import rasterize_svg_to_png
from geomanager.utils.tiles import get_vector_render_layers


class TileFillVectorLayerBlock(FillVectorLayerBlock):
    source_layer = blocks.CharBlock(required=True, label=_("source layer"))


class TileLineVectorLayerBlock(LineVectorLayerBlock):
    source_layer = blocks.CharBlock(required=True, label=_("source layer"))


class TileCircleVectorLayerBlock(CircleVectorLayerBlock):
    source_layer = blocks.CharBlock(required=True, label=_("source layer"))


class TileIconVectorLayerBlock(IconVectorLayerBlock):
    source_layer = blocks.CharBlock(required=True, label=_("source layer"))


class TileTextVectorLayerBlock(TextVectorLayerBlock):
    source_layer = blocks.CharBlock(required=True, label=_("source layer"))


class VectorTileLayer(BaseTileLayer):
    dataset = models.ForeignKey(Dataset, on_delete=models.CASCADE, related_name="vector_tile_layers",
                                verbose_name=_("dataset"))

    render_layers = StreamField([
        ("fill", TileFillVectorLayerBlock(label=_("Polygon Layer"))),
        ("line", TileLineVectorLayerBlock(label=_("Line Layer"))),
        ("circle", TileCircleVectorLayerBlock(label=_("Point Layer"))),
        ("icon", TileIconVectorLayerBlock(label=_("Icon Layer"))),
        ("text", TileTextVectorLayerBlock(label=_("Text Label Layer"))),
    ], use_json_field=True, null=True, blank=True, min_num=1, verbose_name=_("Render Layers"))

    class Meta:
        verbose_name = _("Vector Tile Layer")
        verbose_name_plural = _("Vector Tile Layers")

    panels = [
        FieldPanel("dataset"),
        *BaseTileLayer.panels,
        FieldPanel("render_layers"),
    ]

    def __str__(self):
        return self.title

    @property
    def preview_url(self):
        preview_url = reverse(
            f"geomanager_preview_vector_tile_layer",
            args=[quote(self.dataset.pk), quote(self.pk)],
        )
        return preview_url

    @property
    def layer_config(self):
        tile_url = self.tile_url

        layer_config = {
            "type": "vector",
            "source": {
                "type": "vector",
                "tiles": [tile_url]
            }
        }

        render_layers = get_vector_render_layers(self.render_layers)

        layer_config.update({"render": {"layers": render_layers}})

        return layer_config

    def save(self, *args, **kwargs):
        # remove existing icons for this layer.
        # TODO: Find efficient way to update exising icons, while deleting obsolete ones
        VectorTileLayerIcon.objects.filter(layer=self).delete()

        for render_layer in self.render_layers:
            if render_layer.block_type == "icon":
                icon_image = render_layer.value.get("layout").get("icon_image")
                icon_color = render_layer.value.get("paint").get("icon_color")
                png_bytes = rasterize_svg_to_png(icon_image, fill_color=icon_color)

                if png_bytes:
                    layer_icon = VectorTileLayerIcon(name=icon_image, color=icon_color)
                    layer_icon.file = ContentFile(png_bytes.getvalue(), f"{icon_image}.{icon_color}.png")
                    self.icons.add(layer_icon)

        super().save(*args, **kwargs)


class VectorTileLayerIcon(models.Model):
    layer = ParentalKey(VectorTileLayer, on_delete=models.CASCADE, related_name="icons")
    name = models.CharField(max_length=255)
    color = models.CharField(max_length=100, null=True)
    file = models.FileField(upload_to="vector_tile_icons/")

    def __str__(self):
        return f"{self.layer.title}-{self.name}-{self.color}"



from functools import cached_property

from django.core.validators import MinValueValidator, MaxValueValidator
from django.utils.translation import gettext_lazy as _
from wagtail import blocks
from wagtail.blocks import StructValue
from wagtail_color_panel.blocks import NativeColorBlock
from wagtailiconchooser.blocks import IconChooserBlock


class NavigationItemsBlock(blocks.StructBlock):
    label = blocks.CharBlock(label=_("Label"))
    page = blocks.PageChooserBlock(required=False, label=_("Page"), help_text=_("Internal page to navigate"))
    external_link = blocks.URLBlock(required=False, label=_("External Link"),
                                    help_text=_("External link to navigate to. Used if internal page not provided"))


class QueryParamStaticBlock(blocks.StructBlock):
    key = blocks.CharBlock(label=_("Key"))
    value = blocks.CharBlock(label=_("Value"))


class QueryParamSelectableBlock(blocks.StructBlock):
    SELECTOR_TYPE_CHOICES = (
        ("radio", _("Radio")),
        ("dropdown", _("Dropdown")),
    )
    name = blocks.CharBlock(label=_("name"))
    label = blocks.CharBlock(required=False, label=_("label"))
    type = blocks.ChoiceBlock(choices=SELECTOR_TYPE_CHOICES, default="radio", label=_("Selector Type"))
    options = blocks.ListBlock(blocks.StructBlock([
        ('label', blocks.CharBlock(label=_("label"))),
        ('value', blocks.CharBlock(label=_("value"))),
        ('default', blocks.BooleanBlock(required=False, label=_("default"),
                                        help_text=_("Check to make default option")))]
    ), min_num=1, label=_("Options"))


class InlineLegendBlock(blocks.StructBlock):
    LEGEND_TYPES = (
        ("basic", _("Basic")),
        ("gradient", _("Gradient")),
        ("choropleth", _("Choropleth")),
    )
    type = blocks.ChoiceBlock(choices=LEGEND_TYPES, default="basic", label=_("Legend Type"))
    items = blocks.ListBlock(blocks.StructBlock([
        ('value', blocks.CharBlock(label=_("value"),
                                   help_text=_("Can be a number or text e.g '10' or '10-20' or 'Vegetation'"))),
        ('color', blocks.CharBlock(label=_("color"), help_text=_("Color value e.g rgb(73,73,73) or #494949"))),
    ]
    ), min_num=1, label=_("Legend Items"), )


class InlineIconLegendBlock(blocks.StructBlock):
    items = blocks.ListBlock(blocks.StructBlock([
        ('icon_image', IconChooserBlock(label=_("Icon Image"))),
        ('icon_label', blocks.CharBlock(label=_("Icon Label"), )),
        ('icon_color', NativeColorBlock(required=False, default="#000000", label=_("Icon color"))),
    ]
    ), min_num=1, label=_("Legend Icons"), )


# A filled polygon with an optional stroked border
class FillVectorLayerBlock(blocks.StructBlock):
    paint = blocks.StructBlock([
        ('fill_color', NativeColorBlock(required=False, default="#000000")),
        ('fill_opacity', blocks.FloatBlock(required=False, default=1.0,
                                           validators=[MinValueValidator(0), MaxValueValidator(1)],
                                           )),
        ('fill_outline_color', NativeColorBlock(required=False, default="#000000")),
        ('fill_antialias', blocks.BooleanBlock(required=False, default=True)),
    ], label="Mapbox GL Paint Properties")

    filter = blocks.CharBlock(required=False)
    maxzoom = blocks.IntegerBlock(required=False)
    minzoom = blocks.IntegerBlock(required=False)


# A stroked line
class LineVectorLayerBlock(blocks.StructBlock):
    LINE_CAP_CHOICES = (
        ("butt", "Butt"),
        ("round", "Round"),
        ("square", "Square"),
    )

    LINE_JOIN_CHOICES = (
        ("miter", "Miter"),
        ("bevel", "Bevel"),
        ("round", "Round"),
    )

    paint = blocks.StructBlock([
        ('line_color', NativeColorBlock(required=False, default="#000000")),
        ('line_dasharray', blocks.CharBlock(required=False)),
        ('line_gap_width', blocks.FloatBlock(required=False, validators=[MinValueValidator(0)], default=0.0, )),
        ('line_opacity', blocks.FloatBlock(required=False, validators=[MinValueValidator(0), MaxValueValidator(1)],
                                           default=1.0)),
        ('line_width', blocks.FloatBlock(required=False, validators=[MinValueValidator(0)], default=1.0, )),
        ('line_offset', blocks.FloatBlock(required=False, validators=[MinValueValidator(0)], default=0)),

        # Not implemented here
        # line-gradient - reason => requires source to be geojson, but our source is vector tiles
        # line-pattern
        # line-translate
        # line-translate-anchor
        # line-trim-offset

    ], label="Mapbox GL Paint Properties")

    layout = blocks.StructBlock([
        ('line_cap', blocks.ChoiceBlock(required=False, choices=LINE_CAP_CHOICES, default="butt")),
        ('line_join', blocks.ChoiceBlock(required=False, choices=LINE_JOIN_CHOICES, default="miter")),
        ('line_miter_limit', blocks.FloatBlock(required=False, validators=[MinValueValidator(0)], default=2.0)),
        ('line_round_limit', blocks.FloatBlock(required=False, validators=[MinValueValidator(0)], default=1.05)),
        # Note implemented here
        # line-sort-key
    ], label="Mapbox GL Layout Properties")

    filter = blocks.CharBlock(required=False)
    maxzoom = blocks.IntegerBlock(required=False)
    minzoom = blocks.IntegerBlock(required=False)


SYMBOL_ANCHOR_CHOICES = (
    ("center", "Center"),
    ("left", "Left"),
    ("right", "Right"),
    ("top", "Top"),
    ("bottom", "Bottom"),
    ("top-left", "Top Left"),
    ("top-right", "Top Right"),
    ("bottom-left", "Bottom Left"),
    ("bottom-right", "Bottom Right"),
)

SYMBOL_ALIGNMENT_CHOICES = (
    ("auto", "Auto"),
    ("map", "Map"),
    ("viewport", "Viewport"),
)


# An icon
class IconVectorLayerBlock(blocks.StructBlock):
    ICON_TEXT_FIT_CHOICES = (
        ("none", "None"),
        ("width", "Width"),
        ("height", "Height"),
        ("both", "Both"),
    )

    layout = blocks.StructBlock([
        ('icon_image', IconChooserBlock(label=_("Icon Image"))),
        ('icon_allow_overlap', blocks.BooleanBlock(required=False, default=False)),
        ('icon_anchor', blocks.ChoiceBlock(required=False, choices=SYMBOL_ANCHOR_CHOICES, default="center")),
        ('icon_ignore_placement', blocks.BooleanBlock(required=False, default=False)),
        ('icon_keep_upright', blocks.BooleanBlock(required=False, default=False)),
        ('icon_offset', blocks.CharBlock(required=False)),
        ('icon_optional', blocks.BooleanBlock(required=False, default=False)),
        ('icon_padding', blocks.FloatBlock(required=False, validators=[MinValueValidator(0)], default=2.0)),
        ('icon_pitch_alignment', blocks.ChoiceBlock(required=False, choices=SYMBOL_ALIGNMENT_CHOICES, default="auto")),
        ('icon_rotate', blocks.IntegerBlock(required=False, validators=[MinValueValidator(0), MaxValueValidator(360)],
                                            default=0)),
        ('icon_rotation_alignment', blocks.ChoiceBlock(required=False, choices=SYMBOL_ALIGNMENT_CHOICES, default="auto",
                                                       )),
        ('icon_size', blocks.FloatBlock(required=False, validators=[MinValueValidator(0), MaxValueValidator(1)],
                                        default=1)),
        ('icon_text_fit', blocks.ChoiceBlock(required=False, choices=ICON_TEXT_FIT_CHOICES, default="none")),

        # Not implemented yet
        # icon-text-fit-padding
        # symbol-avoid-edges

    ], label="Mapbox GL Layout Properties")

    paint = blocks.StructBlock([
        ('icon_color', NativeColorBlock(required=False, default="#000000")),
        ('icon_halo_blur', blocks.FloatBlock(required=False, validators=[MinValueValidator(0)], default=0.0)),
        ('icon_halo_color', NativeColorBlock(required=False, default="#000000")),
        ('icon_halo_width', blocks.FloatBlock(required=False, validators=[MinValueValidator(0)], default=0.0)),
        ('icon_opacity', blocks.FloatBlock(required=False, validators=[MinValueValidator(0), MaxValueValidator(1)],
                                           default=1.0)),

        # Not implemented yet
        # icon-translate
        # icon-translate-anchor

    ], label="Mapbox GL Paint Properties")


# Text label
class TextVectorLayerBlock(blocks.StructBlock):
    TEXT_JUSTIFY_CHOICES = (
        ("center", "Center"),
        ("left", "Left"),
        ("right", "Right"),
        ("auto", "Auto"),
    )

    TEXT_TRANSFORM_CHOICES = (
        ("none", "None"),
        ("uppercase", "Uppercase"),
        ("lowercase", "Lowercase"),
    )

    TEXT_TRANSLATE_ANCHOR_CHOICES = (
        ("map", "Map"),
        ("viewport", "Viewport"),
    )

    TEXT_WRITING_MODE_CHOICES = (
        ("horizontal", "Horizontal"),
        ("vertical", "Vertical"),
    )

    SYMBOL_PLACEMENT_CHOICES = (
        ("point", "Point"),
        ("line", "Line"),
        ("line-center", "Line Center"),
    )

    paint = blocks.StructBlock([
        ('text_color', NativeColorBlock(required=False, default="#000000")),
        ('text_halo_blur', blocks.FloatBlock(required=False, validators=[MinValueValidator(0)], default=0.0)),
        ('text_halo_color', NativeColorBlock(required=False, default="#000000")),
        ('text_halo_width', blocks.FloatBlock(required=False, validators=[MinValueValidator(0)], default=0.0)),
        ('text_translate', blocks.CharBlock(required=False)),
        ('text_translate_anchor', blocks.ChoiceBlock(required=False, choices=TEXT_TRANSLATE_ANCHOR_CHOICES,
                                                     default="map")),
    ], label="Mapbox GL Paint Properties")

    layout = blocks.StructBlock([
        ('symbol_placement', blocks.ChoiceBlock(required=False, choices=SYMBOL_PLACEMENT_CHOICES, default="point")),
        ('text_allow_overlap', blocks.BooleanBlock(required=False, default=False)),
        ('text_anchor', blocks.ChoiceBlock(required=False, choices=SYMBOL_ANCHOR_CHOICES, default="center")),
        ('text_field', blocks.CharBlock(required=True)),
        ('text_size', blocks.IntegerBlock(required=False, validators=[MinValueValidator(0)], default=16)),
        ('text_transform', blocks.ChoiceBlock(required=False, choices=TEXT_TRANSFORM_CHOICES, default="none")),
        ('text_ignore_placement', blocks.BooleanBlock(required=False, default=False)),
        ('text_justify', blocks.ChoiceBlock(required=False, choices=TEXT_JUSTIFY_CHOICES, default="center")),
        ('text_keep_upright', blocks.BooleanBlock(required=False, default=False)),
        ('text_letter_spacing', blocks.FloatBlock(required=False, validators=[MinValueValidator(0)], default=0.0)),
        ('text_line_height', blocks.FloatBlock(required=False, validators=[MinValueValidator(0)], default=1.2)),
        ('text_max_angle', blocks.IntegerBlock(required=False,
                                               validators=[MinValueValidator(0), MaxValueValidator(360)], default=45)),
        ('text_max_width', blocks.IntegerBlock(required=False, validators=[MinValueValidator(0)], default=10)),
        ('text_offset', blocks.CharBlock(required=False)),
        ('text_opacity', blocks.FloatBlock(required=False, validators=[MinValueValidator(0), MaxValueValidator(1)],
                                           default=1.0)),
        ('text_padding', blocks.IntegerBlock(required=False, validators=[MinValueValidator(0)], default=2)),
        ('text_pitch_alignment', blocks.ChoiceBlock(required=False, choices=SYMBOL_ALIGNMENT_CHOICES, default="auto")),
        ('text_radial_offset', blocks.IntegerBlock(required=False, validators=[MinValueValidator(0)], default=0)),
        ('text_rotate', blocks.IntegerBlock(required=False, validators=[MinValueValidator(0), MaxValueValidator(360)],
                                            default=0)),
        ('text_rotation_alignment', blocks.ChoiceBlock(required=False, choices=SYMBOL_ALIGNMENT_CHOICES,
                                                       default="auto", )),
        ('text_variable_anchor', blocks.ChoiceBlock(required=False, choices=SYMBOL_ANCHOR_CHOICES)),

        # Not implemented yet
        # symbol-avoid-edges
        # symbol-sort-key
        # symbol-spacing
        # text-font - reason => we should control the fonts used
        # text-optional
        # text-writing-mode

    ], label="Mapbox GL Layout Properties")


# A filled circle
class CircleVectorLayerBlock(blocks.StructBlock):
    paint = blocks.StructBlock([
        ('circle_color', NativeColorBlock(required=False, default="#000000")),
        ('circle_opacity', blocks.FloatBlock(required=False, validators=[MinValueValidator(0), MaxValueValidator(1)],
                                             default=1.0)),
        ('circle_radius', blocks.FloatBlock(required=False, validators=[MinValueValidator(0)], default=5.0,
                                            label=_("circle radius"))),
        ('circle_stroke_color', NativeColorBlock(required=False, default="#000000")),
        ('circle_stroke_width', blocks.FloatBlock(required=False, validators=[MinValueValidator(0)], default=0.0)),

    ], label="Mapbox GL Paint Properties")

    filter = blocks.CharBlock(required=False)
    maxzoom = blocks.IntegerBlock(required=False)
    minzoom = blocks.IntegerBlock(required=False)


TIMESERIES_CHART_TYPES = (
    ("lines", _("Line Chart")),
    ("bars", _("Bar Chart")),
)


class FileLayerPointAnalysisBlock(blocks.StructBlock):
    instance_data_enabled = blocks.BooleanBlock(required=False, default=True, label=_("Show data for point"))
    timeseries_data_enabled = blocks.BooleanBlock(required=False, default=True,
                                                  label=_("Show timeseries data for point"))
    unit = blocks.CharBlock(required=False, label=_("Data unit"))
    timeseries_chart_type = blocks.ChoiceBlock(choices=TIMESERIES_CHART_TYPES, default="bars", label=_("Chart Type"))
    timeseries_chart_color = NativeColorBlock(required=True, default="#367DA4", label=_("Chart Line/Bar Color"))


class FileLayerAreaAnalysisBlock(blocks.StructBlock):
    INSTANCE_VALUE_TYPE_CHOICES = (
        ("mean", _("Mean of pixel values")),
        ("sum", _("Sum of pixel values")),
        ("minmax", _("Minimum, Maximum pixel values")),
        ("minmeanmax", _("Minimum, Mean, Maximum pixel values"))
    )

    TIMESERIES_AGGREGATION_METHODS = (
        ("mean", _("By Mean")),
        ("sum", _("By Sum"))
    )

    instance_data_enabled = blocks.BooleanBlock(required=False, default=True, label=_("Show data for area"))
    instance_value_type = blocks.ChoiceBlock(choices=INSTANCE_VALUE_TYPE_CHOICES, default="mean",
                                             label=_("Area value type"),
                                             help_text=_("The value type that should be displayed"))
    unit = blocks.CharBlock(required=False, label=_("Data unit"))
    timeseries_data_enabled = blocks.BooleanBlock(required=False, default=True,
                                                  label=_("Show timeseries data for area"))
    timeseries_aggregation_method = blocks.ChoiceBlock(choices=TIMESERIES_AGGREGATION_METHODS, default="mean",
                                                       label=_("Area timeseries data aggregation Method"),
                                                       help_text=_("How should the region data be aggregated ?"))

    timeseries_chart_type = blocks.ChoiceBlock(choices=TIMESERIES_CHART_TYPES, default="bars", label=_("Chart Type"))
    timeseries_chart_color = NativeColorBlock(required=True, default="#367DA4", label=_("circle stroke color"))


class LayerMoreInfoStructValue(StructValue):
    @cached_property
    def as_dict(self):
        info = {
            "linkText": self.get("link_text"),
            "linkUrl": self.get("link_url"),
            "text": self.get("text"),
            "isButton": self.get("is_button"),
            "showArrow": self.get("show_arrow"),
        }

        return info


class LayerMoreInfoBlock(blocks.StructBlock):
    link_text = blocks.CharBlock(required=True, label=_("Link text"))
    link_url = blocks.URLBlock(required=True, label=_("Link Url"))
    text = blocks.TextBlock(required=False, label=_("Short description text"))
    is_button = blocks.BooleanBlock(required=False, default=True, label=_("Format link as action button"))
    show_arrow = blocks.BooleanBlock(required=False, default=True, label=_("Show arrow in button"))

    class Meta:
        value_class = LayerMoreInfoStructValue



# Generated by Django 4.1.10 on 2023-10-23 09:15

import django.core.validators
from django.db import migrations, models
import django.db.models.deletion
import wagtail.blocks
import wagtail.fields
import wagtail_color_panel.blocks
import wagtailiconchooser.blocks


class Migration(migrations.Migration):

    dependencies = [
        ('geomanager', '0001_squashed_0024_rastertilelayer_get_time_from_tile_json_and_more'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='areaofinterest',
            options={'verbose_name': 'Area of Interest', 'verbose_name_plural': 'Areas of Interest'},
        ),
        migrations.AlterModelOptions(
            name='category',
            options={'ordering': ['order'], 'verbose_name': 'Category', 'verbose_name_plural': 'Categories'},
        ),
        migrations.AlterModelOptions(
            name='dataset',
            options={'verbose_name': 'Dataset', 'verbose_name_plural': 'Datasets'},
        ),
        migrations.AlterModelOptions(
            name='geomanagersettings',
            options={'verbose_name': 'Geomanager Settings'},
        ),
        migrations.AlterModelOptions(
            name='geomanageruserprofile',
            options={'verbose_name': 'GeoManager User Profile', 'verbose_name_plural': 'GeoManager User Profiles'},
        ),
        migrations.AlterModelOptions(
            name='layerrasterfile',
            options={'ordering': ['time'], 'verbose_name': 'Layer Raster File', 'verbose_name_plural': 'Layer Raster Files'},
        ),
        migrations.AlterModelOptions(
            name='mbtsource',
            options={'verbose_name': 'Basemap Source', 'verbose_name_plural': 'Basemap Sources'},
        ),
        migrations.AlterModelOptions(
            name='metadata',
            options={'verbose_name': 'Metadata', 'verbose_name_plural': 'Metadata'},
        ),
        migrations.AlterModelOptions(
            name='rasterfilelayer',
            options={'verbose_name': 'Raster File Layer', 'verbose_name_plural': 'Raster File Layers'},
        ),
        migrations.AlterModelOptions(
            name='rasterstyle',
            options={'verbose_name': 'Raster Style', 'verbose_name_plural': 'Raster Styles'},
        ),
        migrations.AlterModelOptions(
            name='rastertilelayer',
            options={'verbose_name': 'Raster Tile Layer', 'verbose_name_plural': 'Raster Tile Layers'},
        ),
        migrations.AlterModelOptions(
            name='rasterupload',
            options={'verbose_name': 'Raster Upload', 'verbose_name_plural': 'Raster Uploads'},
        ),
        migrations.AlterModelOptions(
            name='subcategory',
            options={'verbose_name': 'Subcategory', 'verbose_name_plural': 'Subcategories'},
        ),
        migrations.AlterModelOptions(
            name='vectorfilelayer',
            options={'verbose_name': 'Vector File Layer', 'verbose_name_plural': 'Vector File Layers'},
        ),
        migrations.AlterModelOptions(
            name='vectorlayericon',
            options={'verbose_name': 'Vector Layer Icon', 'verbose_name_plural': 'Vector Layer Icons'},
        ),
        migrations.AlterModelOptions(
            name='vectortilelayer',
            options={'verbose_name': 'Vector Tile Layer', 'verbose_name_plural': 'Vector Tile Layers'},
        ),
        migrations.AlterModelOptions(
            name='vectorupload',
            options={'verbose_name': 'Vector Upload', 'verbose_name_plural': 'Vector Uploads'},
        ),
        migrations.AlterModelOptions(
            name='wmslayer',
            options={'verbose_name': 'WMS Layer', 'verbose_name_plural': 'WMS Layers'},
        ),
        migrations.AlterModelOptions(
            name='wmsrequestlayer',
            options={'verbose_name': 'WMS Request Layer', 'verbose_name_plural': 'WMS Request Layers'},
        ),
        migrations.AlterModelOptions(
            name='wmsrequestparam',
            options={'verbose_name': 'WMS Request Parameter', 'verbose_name_plural': 'WMS Request Parameters'},
        ),
        migrations.AlterModelOptions(
            name='wmsrequeststyle',
            options={'verbose_name': 'WMS Request Style', 'verbose_name_plural': 'WMS Request Styles'},
        ),
        migrations.AlterField(
            model_name='dataset',
            name='sub_category',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='geomanager.subcategory', verbose_name='Subcategory'),
        ),
        migrations.AlterField(
            model_name='vectorfilelayer',
            name='render_layers',
            field=wagtail.fields.StreamField([('fill', wagtail.blocks.StructBlock([('paint', wagtail.blocks.StructBlock([('fill_color', wagtail_color_panel.blocks.NativeColorBlock(default='#000000', required=False)), ('fill_opacity', wagtail.blocks.FloatBlock(default=1.0, required=False, validators=[django.core.validators.MinValueValidator(0), django.core.validators.MaxValueValidator(1)])), ('fill_outline_color', wagtail_color_panel.blocks.NativeColorBlock(default='#000000', required=False)), ('fill_antialias', wagtail.blocks.BooleanBlock(default=True, required=False))], label='Mapbox GL Paint Properties')), ('filter', wagtail.blocks.CharBlock(required=False)), ('maxzoom', wagtail.blocks.IntegerBlock(required=False)), ('minzoom', wagtail.blocks.IntegerBlock(required=False))], label='Polygon Layer')), ('line', wagtail.blocks.StructBlock([('paint', wagtail.blocks.StructBlock([('line_color', wagtail_color_panel.blocks.NativeColorBlock(default='#000000', required=False)), ('line_dasharray', wagtail.blocks.CharBlock(required=False)), ('line_gap_width', wagtail.blocks.FloatBlock(default=0.0, required=False, validators=[django.core.validators.MinValueValidator(0)])), ('line_opacity', wagtail.blocks.FloatBlock(default=1.0, required=False, validators=[django.core.validators.MinValueValidator(0), django.core.validators.MaxValueValidator(1)])), ('line_width', wagtail.blocks.FloatBlock(default=1.0, required=False, validators=[django.core.validators.MinValueValidator(0)])), ('line_offset', wagtail.blocks.FloatBlock(default=0, required=False, validators=[django.core.validators.MinValueValidator(0)]))], label='Mapbox GL Paint Properties')), ('layout', wagtail.blocks.StructBlock([('line_cap', wagtail.blocks.ChoiceBlock(choices=[('butt', 'Butt'), ('round', 'Round'), ('square', 'Square')], required=False)), ('line_join', wagtail.blocks.ChoiceBlock(choices=[('miter', 'Miter'), ('bevel', 'Bevel'), ('round', 'Round')], required=False)), ('line_miter_limit', wagtail.blocks.FloatBlock(default=2.0, required=False, validators=[django.core.validators.MinValueValidator(0)])), ('line_round_limit', wagtail.blocks.FloatBlock(default=1.05, required=False, validators=[django.core.validators.MinValueValidator(0)]))], label='Mapbox GL Layout Properties')), ('filter', wagtail.blocks.CharBlock(required=False)), ('maxzoom', wagtail.blocks.IntegerBlock(required=False)), ('minzoom', wagtail.blocks.IntegerBlock(required=False))], label='Line Layer')), ('circle', wagtail.blocks.StructBlock([('paint', wagtail.blocks.StructBlock([('circle_color', wagtail_color_panel.blocks.NativeColorBlock(default='#000000', required=False)), ('circle_opacity', wagtail.blocks.FloatBlock(default=1.0, required=False, validators=[django.core.validators.MinValueValidator(0), django.core.validators.MaxValueValidator(1)])), ('circle_radius', wagtail.blocks.FloatBlock(default=5.0, label='circle radius', required=False, validators=[django.core.validators.MinValueValidator(0)])), ('circle_stroke_color', wagtail_color_panel.blocks.NativeColorBlock(default='#000000', required=False)), ('circle_stroke_width', wagtail.blocks.FloatBlock(default=0.0, required=False, validators=[django.core.validators.MinValueValidator(0)]))], label='Mapbox GL Paint Properties')), ('filter', wagtail.blocks.CharBlock(required=False)), ('maxzoom', wagtail.blocks.IntegerBlock(required=False)), ('minzoom', wagtail.blocks.IntegerBlock(required=False))], label='Point Layer')), ('icon', wagtail.blocks.StructBlock([('layout', wagtail.blocks.StructBlock([('icon_image', wagtailiconchooser.blocks.IconChooserBlock(label='Icon Image')), ('icon_allow_overlap', wagtail.blocks.BooleanBlock(default=False, required=False)), ('icon_anchor', wagtail.blocks.ChoiceBlock(choices=[('center', 'Center'), ('left', 'Left'), ('right', 'Right'), ('top', 'Top'), ('bottom', 'Bottom'), ('top-left', 'Top Left'), ('top-right', 'Top Right'), ('bottom-left', 'Bottom Left'), ('bottom-right', 'Bottom Right')], required=False)), ('icon_ignore_placement', wagtail.blocks.BooleanBlock(default=False, required=False)), ('icon_keep_upright', wagtail.blocks.BooleanBlock(default=False, required=False)), ('icon_offset', wagtail.blocks.CharBlock(required=False)), ('icon_optional', wagtail.blocks.BooleanBlock(default=False, required=False)), ('icon_padding', wagtail.blocks.FloatBlock(default=2.0, required=False, validators=[django.core.validators.MinValueValidator(0)])), ('icon_pitch_alignment', wagtail.blocks.ChoiceBlock(choices=[('auto', 'Auto'), ('map', 'Map'), ('viewport', 'Viewport')], required=False)), ('icon_rotate', wagtail.blocks.IntegerBlock(default=0, required=False, validators=[django.core.validators.MinValueValidator(0), django.core.validators.MaxValueValidator(360)])), ('icon_rotation_alignment', wagtail.blocks.ChoiceBlock(choices=[('auto', 'Auto'), ('map', 'Map'), ('viewport', 'Viewport')], required=False)), ('icon_size', wagtail.blocks.FloatBlock(default=1, required=False, validators=[django.core.validators.MinValueValidator(0), django.core.validators.MaxValueValidator(1)])), ('icon_text_fit', wagtail.blocks.ChoiceBlock(choices=[('none', 'None'), ('width', 'Width'), ('height', 'Height'), ('both', 'Both')], required=False))], label='Mapbox GL Layout Properties')), ('paint', wagtail.blocks.StructBlock([('icon_color', wagtail_color_panel.blocks.NativeColorBlock(default='#000000', required=False)), ('icon_halo_blur', wagtail.blocks.FloatBlock(default=0.0, required=False, validators=[django.core.validators.MinValueValidator(0)])), ('icon_halo_color', wagtail_color_panel.blocks.NativeColorBlock(default='#000000', required=False)), ('icon_halo_width', wagtail.blocks.FloatBlock(default=0.0, required=False, validators=[django.core.validators.MinValueValidator(0)])), ('icon_opacity', wagtail.blocks.FloatBlock(default=1.0, required=False, validators=[django.core.validators.MinValueValidator(0), django.core.validators.MaxValueValidator(1)]))], label='Mapbox GL Paint Properties'))], label='Icon Layer')), ('text', wagtail.blocks.StructBlock([('paint', wagtail.blocks.StructBlock([('text_color', wagtail_color_panel.blocks.NativeColorBlock(default='#000000', required=False)), ('text_halo_blur', wagtail.blocks.FloatBlock(default=0.0, required=False, validators=[django.core.validators.MinValueValidator(0)])), ('text_halo_color', wagtail_color_panel.blocks.NativeColorBlock(default='#000000', required=False)), ('text_halo_width', wagtail.blocks.FloatBlock(default=0.0, required=False, validators=[django.core.validators.MinValueValidator(0)])), ('text_translate', wagtail.blocks.CharBlock(required=False)), ('text_translate_anchor', wagtail.blocks.ChoiceBlock(choices=[('map', 'Map'), ('viewport', 'Viewport')], required=False))], label='Mapbox GL Paint Properties')), ('layout', wagtail.blocks.StructBlock([('symbol_placement', wagtail.blocks.ChoiceBlock(choices=[('point', 'Point'), ('line', 'Line'), ('line-center', 'Line Center')], required=False)), ('text_allow_overlap', wagtail.blocks.BooleanBlock(default=False, required=False)), ('text_anchor', wagtail.blocks.ChoiceBlock(choices=[('center', 'Center'), ('left', 'Left'), ('right', 'Right'), ('top', 'Top'), ('bottom', 'Bottom'), ('top-left', 'Top Left'), ('top-right', 'Top Right'), ('bottom-left', 'Bottom Left'), ('bottom-right', 'Bottom Right')], required=False)), ('text_field', wagtail.blocks.CharBlock(required=True)), ('text_size', wagtail.blocks.IntegerBlock(default=16, required=False, validators=[django.core.validators.MinValueValidator(0)])), ('text_transform', wagtail.blocks.ChoiceBlock(choices=[('none', 'None'), ('uppercase', 'Uppercase'), ('lowercase', 'Lowercase')], required=False)), ('text_ignore_placement', wagtail.blocks.BooleanBlock(default=False, required=False)), ('text_justify', wagtail.blocks.ChoiceBlock(choices=[('center', 'Center'), ('left', 'Left'), ('right', 'Right'), ('auto', 'Auto')], required=False)), ('text_keep_upright', wagtail.blocks.BooleanBlock(default=False, required=False)), ('text_letter_spacing', wagtail.blocks.FloatBlock(default=0.0, required=False, validators=[django.core.validators.MinValueValidator(0)])), ('text_line_height', wagtail.blocks.FloatBlock(default=1.2, required=False, validators=[django.core.validators.MinValueValidator(0)])), ('text_max_angle', wagtail.blocks.IntegerBlock(default=45, required=False, validators=[django.core.validators.MinValueValidator(0), django.core.validators.MaxValueValidator(360)])), ('text_max_width', wagtail.blocks.IntegerBlock(default=10, required=False, validators=[django.core.validators.MinValueValidator(0)])), ('text_offset', wagtail.blocks.CharBlock(required=False)), ('text_opacity', wagtail.blocks.FloatBlock(default=1.0, required=False, validators=[django.core.validators.MinValueValidator(0), django.core.validators.MaxValueValidator(1)])), ('text_padding', wagtail.blocks.IntegerBlock(default=2, required=False, validators=[django.core.validators.MinValueValidator(0)])), ('text_pitch_alignment', wagtail.blocks.ChoiceBlock(choices=[('auto', 'Auto'), ('map', 'Map'), ('viewport', 'Viewport')], required=False)), ('text_radial_offset', wagtail.blocks.IntegerBlock(default=0, required=False, validators=[django.core.validators.MinValueValidator(0)])), ('text_rotate', wagtail.blocks.IntegerBlock(default=0, required=False, validators=[django.core.validators.MinValueValidator(0), django.core.validators.MaxValueValidator(360)])), ('text_rotation_alignment', wagtail.blocks.ChoiceBlock(choices=[('auto', 'Auto'), ('map', 'Map'), ('viewport', 'Viewport')], required=False)), ('text_variable_anchor', wagtail.blocks.ChoiceBlock(choices=[('center', 'Center'), ('left', 'Left'), ('right', 'Right'), ('top', 'Top'), ('bottom', 'Bottom'), ('top-left', 'Top Left'), ('top-right', 'Top Right'), ('bottom-left', 'Bottom Left'), ('bottom-right', 'Bottom Right')], required=False))], label='Mapbox GL Layout Properties'))], label='Text Label Layer'))], blank=True, null=True, use_json_field=True, verbose_name='Render Layers'),
        ),
        migrations.AlterField(
            model_name='vectortilelayer',
            name='render_layers',
            field=wagtail.fields.StreamField([('fill', wagtail.blocks.StructBlock([('paint', wagtail.blocks.StructBlock([('fill_color', wagtail_color_panel.blocks.NativeColorBlock(default='#000000', required=False)), ('fill_opacity', wagtail.blocks.FloatBlock(default=1.0, required=False, validators=[django.core.validators.MinValueValidator(0), django.core.validators.MaxValueValidator(1)])), ('fill_outline_color', wagtail_color_panel.blocks.NativeColorBlock(default='#000000', required=False)), ('fill_antialias', wagtail.blocks.BooleanBlock(default=True, required=False))], label='Mapbox GL Paint Properties')), ('filter', wagtail.blocks.CharBlock(required=False)), ('maxzoom', wagtail.blocks.IntegerBlock(required=False)), ('minzoom', wagtail.blocks.IntegerBlock(required=False)), ('source_layer', wagtail.blocks.CharBlock(label='source layer', required=True))], label='Polygon Layer')), ('line', wagtail.blocks.StructBlock([('paint', wagtail.blocks.StructBlock([('line_color', wagtail_color_panel.blocks.NativeColorBlock(default='#000000', required=False)), ('line_dasharray', wagtail.blocks.CharBlock(required=False)), ('line_gap_width', wagtail.blocks.FloatBlock(default=0.0, required=False, validators=[django.core.validators.MinValueValidator(0)])), ('line_opacity', wagtail.blocks.FloatBlock(default=1.0, required=False, validators=[django.core.validators.MinValueValidator(0), django.core.validators.MaxValueValidator(1)])), ('line_width', wagtail.blocks.FloatBlock(default=1.0, required=False, validators=[django.core.validators.MinValueValidator(0)])), ('line_offset', wagtail.blocks.FloatBlock(default=0, required=False, validators=[django.core.validators.MinValueValidator(0)]))], label='Mapbox GL Paint Properties')), ('layout', wagtail.blocks.StructBlock([('line_cap', wagtail.blocks.ChoiceBlock(choices=[('butt', 'Butt'), ('round', 'Round'), ('square', 'Square')], required=False)), ('line_join', wagtail.blocks.ChoiceBlock(choices=[('miter', 'Miter'), ('bevel', 'Bevel'), ('round', 'Round')], required=False)), ('line_miter_limit', wagtail.blocks.FloatBlock(default=2.0, required=False, validators=[django.core.validators.MinValueValidator(0)])), ('line_round_limit', wagtail.blocks.FloatBlock(default=1.05, required=False, validators=[django.core.validators.MinValueValidator(0)]))], label='Mapbox GL Layout Properties')), ('filter', wagtail.blocks.CharBlock(required=False)), ('maxzoom', wagtail.blocks.IntegerBlock(required=False)), ('minzoom', wagtail.blocks.IntegerBlock(required=False)), ('source_layer', wagtail.blocks.CharBlock(label='source layer', required=True))], label='Line Layer')), ('circle', wagtail.blocks.StructBlock([('paint', wagtail.blocks.StructBlock([('circle_color', wagtail_color_panel.blocks.NativeColorBlock(default='#000000', required=False)), ('circle_opacity', wagtail.blocks.FloatBlock(default=1.0, required=False, validators=[django.core.validators.MinValueValidator(0), django.core.validators.MaxValueValidator(1)])), ('circle_radius', wagtail.blocks.FloatBlock(default=5.0, label='circle radius', required=False, validators=[django.core.validators.MinValueValidator(0)])), ('circle_stroke_color', wagtail_color_panel.blocks.NativeColorBlock(default='#000000', required=False)), ('circle_stroke_width', wagtail.blocks.FloatBlock(default=0.0, required=False, validators=[django.core.validators.MinValueValidator(0)]))], label='Mapbox GL Paint Properties')), ('filter', wagtail.blocks.CharBlock(required=False)), ('maxzoom', wagtail.blocks.IntegerBlock(required=False)), ('minzoom', wagtail.blocks.IntegerBlock(required=False)), ('source_layer', wagtail.blocks.CharBlock(label='source layer', required=True))], label='Point Layer')), ('icon', wagtail.blocks.StructBlock([('layout', wagtail.blocks.StructBlock([('icon_image', wagtailiconchooser.blocks.IconChooserBlock(label='Icon Image')), ('icon_allow_overlap', wagtail.blocks.BooleanBlock(default=False, required=False)), ('icon_anchor', wagtail.blocks.ChoiceBlock(choices=[('center', 'Center'), ('left', 'Left'), ('right', 'Right'), ('top', 'Top'), ('bottom', 'Bottom'), ('top-left', 'Top Left'), ('top-right', 'Top Right'), ('bottom-left', 'Bottom Left'), ('bottom-right', 'Bottom Right')], required=False)), ('icon_ignore_placement', wagtail.blocks.BooleanBlock(default=False, required=False)), ('icon_keep_upright', wagtail.blocks.BooleanBlock(default=False, required=False)), ('icon_offset', wagtail.blocks.CharBlock(required=False)), ('icon_optional', wagtail.blocks.BooleanBlock(default=False, required=False)), ('icon_padding', wagtail.blocks.FloatBlock(default=2.0, required=False, validators=[django.core.validators.MinValueValidator(0)])), ('icon_pitch_alignment', wagtail.blocks.ChoiceBlock(choices=[('auto', 'Auto'), ('map', 'Map'), ('viewport', 'Viewport')], required=False)), ('icon_rotate', wagtail.blocks.IntegerBlock(default=0, required=False, validators=[django.core.validators.MinValueValidator(0), django.core.validators.MaxValueValidator(360)])), ('icon_rotation_alignment', wagtail.blocks.ChoiceBlock(choices=[('auto', 'Auto'), ('map', 'Map'), ('viewport', 'Viewport')], required=False)), ('icon_size', wagtail.blocks.FloatBlock(default=1, required=False, validators=[django.core.validators.MinValueValidator(0), django.core.validators.MaxValueValidator(1)])), ('icon_text_fit', wagtail.blocks.ChoiceBlock(choices=[('none', 'None'), ('width', 'Width'), ('height', 'Height'), ('both', 'Both')], required=False))], label='Mapbox GL Layout Properties')), ('paint', wagtail.blocks.StructBlock([('icon_color', wagtail_color_panel.blocks.NativeColorBlock(default='#000000', required=False)), ('icon_halo_blur', wagtail.blocks.FloatBlock(default=0.0, required=False, validators=[django.core.validators.MinValueValidator(0)])), ('icon_halo_color', wagtail_color_panel.blocks.NativeColorBlock(default='#000000', required=False)), ('icon_halo_width', wagtail.blocks.FloatBlock(default=0.0, required=False, validators=[django.core.validators.MinValueValidator(0)])), ('icon_opacity', wagtail.blocks.FloatBlock(default=1.0, required=False, validators=[django.core.validators.MinValueValidator(0), django.core.validators.MaxValueValidator(1)]))], label='Mapbox GL Paint Properties')), ('source_layer', wagtail.blocks.CharBlock(label='source layer', required=True))], label='Icon Layer')), ('text', wagtail.blocks.StructBlock([('paint', wagtail.blocks.StructBlock([('text_color', wagtail_color_panel.blocks.NativeColorBlock(default='#000000', required=False)), ('text_halo_blur', wagtail.blocks.FloatBlock(default=0.0, required=False, validators=[django.core.validators.MinValueValidator(0)])), ('text_halo_color', wagtail_color_panel.blocks.NativeColorBlock(default='#000000', required=False)), ('text_halo_width', wagtail.blocks.FloatBlock(default=0.0, required=False, validators=[django.core.validators.MinValueValidator(0)])), ('text_translate', wagtail.blocks.CharBlock(required=False)), ('text_translate_anchor', wagtail.blocks.ChoiceBlock(choices=[('map', 'Map'), ('viewport', 'Viewport')], required=False))], label='Mapbox GL Paint Properties')), ('layout', wagtail.blocks.StructBlock([('symbol_placement', wagtail.blocks.ChoiceBlock(choices=[('point', 'Point'), ('line', 'Line'), ('line-center', 'Line Center')], required=False)), ('text_allow_overlap', wagtail.blocks.BooleanBlock(default=False, required=False)), ('text_anchor', wagtail.blocks.ChoiceBlock(choices=[('center', 'Center'), ('left', 'Left'), ('right', 'Right'), ('top', 'Top'), ('bottom', 'Bottom'), ('top-left', 'Top Left'), ('top-right', 'Top Right'), ('bottom-left', 'Bottom Left'), ('bottom-right', 'Bottom Right')], required=False)), ('text_field', wagtail.blocks.CharBlock(required=True)), ('text_size', wagtail.blocks.IntegerBlock(default=16, required=False, validators=[django.core.validators.MinValueValidator(0)])), ('text_transform', wagtail.blocks.ChoiceBlock(choices=[('none', 'None'), ('uppercase', 'Uppercase'), ('lowercase', 'Lowercase')], required=False)), ('text_ignore_placement', wagtail.blocks.BooleanBlock(default=False, required=False)), ('text_justify', wagtail.blocks.ChoiceBlock(choices=[('center', 'Center'), ('left', 'Left'), ('right', 'Right'), ('auto', 'Auto')], required=False)), ('text_keep_upright', wagtail.blocks.BooleanBlock(default=False, required=False)), ('text_letter_spacing', wagtail.blocks.FloatBlock(default=0.0, required=False, validators=[django.core.validators.MinValueValidator(0)])), ('text_line_height', wagtail.blocks.FloatBlock(default=1.2, required=False, validators=[django.core.validators.MinValueValidator(0)])), ('text_max_angle', wagtail.blocks.IntegerBlock(default=45, required=False, validators=[django.core.validators.MinValueValidator(0), django.core.validators.MaxValueValidator(360)])), ('text_max_width', wagtail.blocks.IntegerBlock(default=10, required=False, validators=[django.core.validators.MinValueValidator(0)])), ('text_offset', wagtail.blocks.CharBlock(required=False)), ('text_opacity', wagtail.blocks.FloatBlock(default=1.0, required=False, validators=[django.core.validators.MinValueValidator(0), django.core.validators.MaxValueValidator(1)])), ('text_padding', wagtail.blocks.IntegerBlock(default=2, required=False, validators=[django.core.validators.MinValueValidator(0)])), ('text_pitch_alignment', wagtail.blocks.ChoiceBlock(choices=[('auto', 'Auto'), ('map', 'Map'), ('viewport', 'Viewport')], required=False)), ('text_radial_offset', wagtail.blocks.IntegerBlock(default=0, required=False, validators=[django.core.validators.MinValueValidator(0)])), ('text_rotate', wagtail.blocks.IntegerBlock(default=0, required=False, validators=[django.core.validators.MinValueValidator(0), django.core.validators.MaxValueValidator(360)])), ('text_rotation_alignment', wagtail.blocks.ChoiceBlock(choices=[('auto', 'Auto'), ('map', 'Map'), ('viewport', 'Viewport')], required=False)), ('text_variable_anchor', wagtail.blocks.ChoiceBlock(choices=[('center', 'Center'), ('left', 'Left'), ('right', 'Right'), ('top', 'Top'), ('bottom', 'Bottom'), ('top-left', 'Top Left'), ('top-right', 'Top Right'), ('bottom-left', 'Bottom Left'), ('bottom-right', 'Bottom Right')], required=False))], label='Mapbox GL Layout Properties')), ('source_layer', wagtail.blocks.CharBlock(label='source layer', required=True))], label='Text Label Layer'))], blank=True, null=True, use_json_field=True, verbose_name='Render Layers'),
        ),
    ]



from .auth import (
    RegisterView,
    ResetPasswordView
)
from .core import get_mapviewer_config
from .nextjs import map_view
from .raster_file import (
    upload_raster_file,
    publish_raster,
    delete_raster_upload,
    RasterTileView,
    preview_raster_layers
)
from .stations import load_stations, preview_stations
from .tile_gl import tile_gl, tile_json_gl, style_json_gl
from .vector_file import (
    upload_vector_file,
    publish_vector,
    delete_vector_upload,
    preview_vector_layers,
    VectorTileView,
    GeoJSONPgTableView
)
from .wms import preview_wms_layers



from django.db import models
from django.utils.translation import gettext_lazy as _
from django_extensions.db.models import TimeStampedModel
from modelcluster.fields import ParentalKey
from modelcluster.models import ClusterableModel
from wagtail.admin.panels import FieldPanel, FieldRowPanel, MultiFieldPanel, InlinePanel
from wagtail.fields import StreamField
from wagtail.images.blocks import ImageChooserBlock
from wagtail.images.models import Image
from wagtail.models import Orderable

from geomanager.blocks import (
    InlineLegendBlock,
    LayerMoreInfoBlock,
    QueryParamSelectableBlock
)
from geomanager.models.core import Dataset, BaseLayer


class WmsLayer(TimeStampedModel, ClusterableModel, BaseLayer):
    OUTPUT_FORMATS = (
        ("image/png", "PNG"),
        ("image/jpeg", "JPEG"),
        ("image/gif", "GIF"),
    )

    VERSION_CHOICES = (
        ("1.1.1", "1.1.1"),
        ("1.3.0", "1.3.0"),
    )

    DATE_FORMAT_CHOICES = (
        ("yyyy-mm-dd HH:MM", _("Hourly - (E.g 2023-01-01 00:00)")),
        ("yyyy-mm-dd", _("Daily - (E.g 2023-01-01)")),
        ("yyyy-mm", _("Monthly - (E.g 2023-01)")),
        ("mmmm yyyy", _("Monthly - (E.g January 2023)")),
        ("pentadal", _("Pentadal - (E.g Jan 2023 - P1 1-5th)"))
    )

    dataset = models.ForeignKey(Dataset, on_delete=models.CASCADE, related_name="wms_layers", verbose_name=_("dataset"))
    request_time_from_capabilities = models.BooleanField(default=True, verbose_name=_("Request time from capabilities"),
                                                         help_text=_("Get list of available times for this layer"
                                                                     " from getCapabilities"), )
    base_url = models.CharField(max_length=500, verbose_name=_("base url for WMS"), )
    version = models.CharField(max_length=50, default="1.1.1", choices=VERSION_CHOICES, verbose_name=_("WMS Version"))
    width = models.IntegerField(default=256, verbose_name=_("Pixel Width"),
                                help_text=_("The size of the map image in pixels along the i axis"), )
    height = models.IntegerField(default=256, verbose_name=_("Pixel Height"),
                                 help_text=_("The size of the map image in pixels along the j axis"), )
    transparent = models.BooleanField(default=True, verbose_name=_("Transparency"),
                                      help_text=_("Ability of underlying maps to be visible or not"), )
    srs = models.CharField(max_length=50, default="EPSG:3857", verbose_name=_("Spatial Reference System"),
                           help_text=_("WMS Spatial Reference e.g EPSG:3857"), )
    format = models.CharField(max_length=50, default="image/png", choices=OUTPUT_FORMATS,
                              verbose_name=_("Output Format"),
                              help_text=_(
                                  "Allowed map formats are either “picture” formats or “graphic element” formats."), )
    wms_query_params_selectable = StreamField([
        ('param', QueryParamSelectableBlock(label=_("Query Parameter")))
    ], use_json_field=True, null=True, blank=True, verbose_name=_("WMS Query Params With selectable Options"),
        help_text=_(
            "This should provide a list of options that users can choose to change the query parameter of the url"))

    params_selectors_side_by_side = models.BooleanField(default=False,
                                                        verbose_name=_("Arrange Param Selectors side by side"))
    legend = StreamField([
        ('legend', InlineLegendBlock(label=_("Custom Legend")),),
        ('legend_image', ImageChooserBlock(label=_("Custom Image")),),
    ], use_json_field=True, null=True, blank=True, max_num=1, verbose_name=_("Legend"), )
    date_format = models.CharField(max_length=100, choices=DATE_FORMAT_CHOICES, blank=True, null=True)
    custom_get_capabilities_url = models.URLField(blank=True, null=True, verbose_name=_("Get Capabilities URL"),
                                                  help_text=_("Alternative URL for the GetCapabilities request. "
                                                              "Used when 'Request time from capabilities' "
                                                              "option is checked. Leave blank to use base url."))
    get_capabilities_layer_name = models.CharField(max_length=255, blank=True, null=True,
                                                   verbose_name=_("Get Capabilities Layer Name"),
                                                   help_text=_("Alternative layer name for the GetCapabilities request"
                                                               ))
    more_info = StreamField([
        ('more_info', LayerMoreInfoBlock(label=_("Info link")),),
    ], block_counts={
        'more_info': {'max_num': 1},
    }, use_json_field=True, null=True, blank=True, max_num=1, verbose_name=_("More Info"), )

    class Meta:
        verbose_name = _("WMS Layer")
        verbose_name_plural = _("WMS Layers")

    panels = [
        FieldPanel("dataset"),
        FieldPanel("title"),
        FieldPanel("default"),
        FieldPanel("date_format"),
        MultiFieldPanel([
            FieldPanel("base_url"),
            FieldPanel("version"),
            FieldRowPanel([
                FieldPanel("width", classname="col6"),
                FieldPanel("height", classname="col6"),
            ]),
            FieldPanel("transparent"),
            FieldPanel("srs"),
            FieldPanel("format"),
            InlinePanel("wms_request_layers", heading=_("WMS Request Layers"),
                        label=_("WMS Request Layer"), min_num=1),
            InlinePanel("wms_request_styles", heading=_("WMS Request Styles"),
                        label=_("WMS Request Style")),
            InlinePanel("wms_request_params", heading=_("WMS Request Additional Parameters"),
                        label=_("WMS Request Param")),
            FieldPanel("wms_query_params_selectable"),
        ], heading=_("WMS GetMap Configuration")),
        MultiFieldPanel([
            FieldPanel("request_time_from_capabilities"),
            FieldPanel("custom_get_capabilities_url"),
            FieldPanel("get_capabilities_layer_name"),
        ], heading=_("WMS GetCapabilities Configuration")),
        FieldPanel("params_selectors_side_by_side"),
        FieldPanel("legend"),
        FieldPanel("more_info"),
    ]

    def __str__(self):
        return self.title

    def get_selectable_params(self):
        params = {}
        if self.wms_query_params_selectable:
            for query_param in self.wms_query_params_selectable:
                data = query_param.block.get_api_representation(query_param.value)
                val = f"{data.get('name')}"
                params.update({val: data})
        return params

    def get_wms_params(self):
        params = {
            "SERVICE": "WMS",
            "VERSION": self.version,
            "REQUEST": "GetMap",
            "TRANSPARENT": self.transparent,
            "LAYERS": ",".join([layer.name for layer in self.wms_request_layers.all()]),
            "STYLES": ",".join([style.name for style in self.wms_request_styles.all()]),
            "BBOX": "{bbox-epsg-3857}",
            "WIDTH": self.width,
            "HEIGHT": self.height,
            "FORMAT": self.format,
        }

        if self.version == "1.3.0":
            params.update({"CRS": self.srs})
        else:
            params.update({"SRS": self.srs})

        extra_params = {param.name: param.value for param in self.wms_request_params.all()}
        params.update(**extra_params)

        return params

    @property
    def get_map_url(self):
        params = self.get_wms_params()
        selectable_params = self.get_selectable_params()

        for key, val in selectable_params.items():
            key_val = key
            if key.upper() in params:
                key_val = key.upper()
            params.update({key_val: f"{{{key}}}"})

        query_str = '&'.join([f"{key}={value}" for key, value in params.items()])
        request_url = f"{self.base_url}?{query_str}"

        return request_url

    def get_selectable_params_config(self):
        selectable_params = self.get_selectable_params()
        config = []

        for key, param_config in selectable_params.items():
            param_config = {
                "key": key,
                "required": True,
                "type": param_config.get("type"),
                "options": param_config.get("options"),
                "sentence": f"{param_config.get('label') or key} {{selector}}",
            }

            config.append(param_config)

        return config

    @property
    def get_capabilities_url(self):
        if self.request_time_from_capabilities:
            capabilities_url = self.base_url

            if self.custom_get_capabilities_url:
                capabilities_url = self.custom_get_capabilities_url

            params = {
                "SERVICE": "WMS",
                "VERSION": self.version,
                "REQUEST": "GetCapabilities",
            }
            query_str = '&'.join([f"{key}={value}" for key, value in params.items()])
            request_url = f"{capabilities_url}?{query_str}"
            return request_url
        return None

    @property
    def layer_config(self):
        wms_url = self.get_map_url
        if self.dataset.multi_temporal:
            wms_url = f"{wms_url}&time={{time}}"

        layer_config = {
            "type": "raster",
            "source": {
                "type": "raster",
                "tiles": [wms_url]
            }
        }

        return layer_config

    @property
    def params(self):
        params = {}
        if self.dataset.multi_temporal:
            params.update({"time": ""})

        selector_config = self.get_selectable_params_config()

        for selector_param in selector_config:
            default = None
            for option in selector_param.get("options"):
                if option.get("default"):
                    default = option.get("value")
                    break
            if not default:
                default = selector_param.get("options")[0].get("value")
            params.update({selector_param.get("key"): default})

        return params

    @property
    def param_selector_config(self):
        config = []

        if self.dataset.multi_temporal:
            time_config = {
                "key": "time",
                "required": True,
                "sentence": "{selector}",
                "type": "datetime",
                "availableDates": [],
            }

            if self.date_format:
                if self.date_format == "pentadal":
                    time_config.update({
                        "dateFormat": {"currentTime": "mmm yyyy", "asPeriod": "pentadal"},
                    })
                else:
                    time_config.update({
                        "dateFormat": {"currentTime": self.date_format},
                    })
            else:
                time_config.update({
                    "dateFormat": {"currentTime": "yyyy-mm-dd HH:MM"},
                })

            config.append(time_config)
        selectable_params_config = self.get_selectable_params_config()

        config.extend(selectable_params_config)

        return config

    def get_legend_config(self, request):

        # default config
        config = {
            "type": "basic",
            "items": []
        }

        legend_block = self.legend

        # only one legend block entry is expected
        if legend_block:
            legend_block = legend_block[0]

        if legend_block:
            if isinstance(legend_block.value, Image):
                image_url = legend_block.value.file.url
                if request:
                    image_url = request.build_absolute_uri(image_url)
                config.update({"type": "image", "imageUrl": image_url})
                return config

            data = legend_block.block.get_api_representation(legend_block.value)

            config.update({"type": data.get("type")})

            for item in data.get("items"):
                config["items"].append({
                    "name": item.get("value"),
                    "color": item.get("color")
                })

        return config

    @property
    def layer_name(self):
        return self.wms_request_layers.all()[0].name

    def get_analysis_config(self):
        analysis_config = []

        return analysis_config

    def get_more_info(self):
        more_info = self.more_info

        return {}


class WmsRequestLayer(Orderable):
    layer = ParentalKey(WmsLayer, on_delete=models.CASCADE, related_name="wms_request_layers",
                        verbose_name=_("WMS Request Layers"))
    name = models.CharField(max_length=250, null=False, blank=False,
                            verbose_name=_("name"),
                            help_text=_("WMS Layer is requested by using this "
                                        "name in the LAYERS parameter of a "
                                        "GetMap request."))

    class Meta:
        verbose_name = _("WMS Request Layer")
        verbose_name_plural = _("WMS Request Layers")


class WmsRequestStyle(Orderable):
    layer = ParentalKey(WmsLayer, on_delete=models.CASCADE,
                        related_name="wms_request_styles", verbose_name=_("WMS Request Styles"))
    name = models.CharField(max_length=250, null=False, blank=False,
                            verbose_name=_("name"),
                            help_text=_("The style's Name is used in the Map request STYLES parameter"))

    class Meta:
        verbose_name = _("WMS Request Style")
        verbose_name_plural = _("WMS Request Styles")


class WmsRequestParam(Orderable):
    layer = ParentalKey(WmsLayer, on_delete=models.CASCADE, related_name="wms_request_params",
                        verbose_name=_("WMS Requests Additional Parameters", ))
    name = models.CharField(max_length=250, null=False, blank=False, verbose_name=_("name"),
                            help_text=_("Name of the parameter"))
    value = models.CharField(max_length=250, null=False, blank=False, help_text=_("Value of the parameter"))

    class Meta:
        verbose_name = _("WMS Request Parameter")
        verbose_name_plural = _("WMS Request Parameters")



from django_filters.rest_framework import DjangoFilterBackend
from django_large_image.rest import LargeImageFileDetailMixin
from rest_framework import mixins, viewsets

from geomanager.models import LayerRasterFile
from geomanager.serializers.raster_file import LayerRasterFileSerializer


class RasterLayerRasterFileDetailViewSet(mixins.ListModelMixin, viewsets.GenericViewSet, LargeImageFileDetailMixin, ):
    queryset = LayerRasterFile.objects.all()
    serializer_class = LayerRasterFileSerializer
    filter_backends = [DjangoFilterBackend]
    filterset_fields = ["layer"]

    FILE_FIELD_NAME = "file"



import json

from django.shortcuts import get_object_or_404, render
from wagtail.admin.auth import (
    user_passes_test,
    user_has_any_page_permission
)
from wagtail_modeladmin.helpers import AdminURLHelper

from geomanager.models import (
    Dataset
)
from geomanager.models.vector_tile import VectorTileLayer, VectorTileLayerIcon
from geomanager.serializers.vector_tile import VectorTileLayerSerializer
from geomanager.utils import UUIDEncoder


@user_passes_test(user_has_any_page_permission)
def preview_vector_tile_layers(request, dataset_id, layer_id=None):
    dataset = get_object_or_404(Dataset, pk=dataset_id)

    dataset_admin_helper = AdminURLHelper(Dataset)
    dataset_list_url = dataset_admin_helper.get_action_url("index")

    vector_tile_layer_admin_helper = AdminURLHelper(VectorTileLayer)
    vector_tile_layer_list_url = vector_tile_layer_admin_helper.get_action_url("index")

    layer = None
    if layer_id:
        layer = VectorTileLayer.objects.get(pk=layer_id)

    dataset_layers = VectorTileLayerSerializer(dataset.vector_tile_layers, many=True).data

    # get icon images for the dataset vector tile layers, if any
    icon_images = []
    layers_id = [layer.get("id") for layer in dataset_layers]
    for icon in VectorTileLayerIcon.objects.filter(layer__in=layers_id):
        icon_images.append({"name": icon.name, "url": request.build_absolute_uri(icon.file.url)})

    context = {
        "dataset": dataset,
        "selected_layer": layer,
        "datasets_index_url": dataset_list_url,
        "vector_tile_layer_list_url": vector_tile_layer_list_url,
        "dataset_layers": json.dumps(dataset_layers, cls=UUIDEncoder),
        "icon_images": icon_images
    }

    return render(request, 'geomanager/vector_tile_layer_preview.html', context)



# Generated by Django 4.1.10 on 2023-10-02 09:50

import django.core.validators
from django.db import migrations, models
import django.db.models.deletion
import django_extensions.db.fields
import modelcluster.fields
import uuid
import wagtail.blocks
import wagtail.fields
import wagtail.images.blocks
import wagtail_color_panel.blocks
import wagtailiconchooser.blocks


class Migration(migrations.Migration):

    dependencies = [
        ('geomanager', '0020_tmslayer'),
    ]

    operations = [
        migrations.CreateModel(
            name='RasterFileLayer',
            fields=[
                ('created', django_extensions.db.fields.CreationDateTimeField(auto_now_add=True, verbose_name='created')),
                ('modified', django_extensions.db.fields.ModificationDateTimeField(auto_now=True, verbose_name='modified')),
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('title', models.CharField(help_text='layer title', max_length=255, verbose_name='title')),
                ('default', models.BooleanField(default=False, help_text='Is Default Layer', verbose_name='default')),
                ('analysis', wagtail.fields.StreamField([('point_analysis', wagtail.blocks.StructBlock([('instance_data_enabled', wagtail.blocks.BooleanBlock(default=True, label='Show data for point', required=False)), ('timeseries_data_enabled', wagtail.blocks.BooleanBlock(default=True, label='Show timeseries data for point', required=False)), ('unit', wagtail.blocks.CharBlock(label='Data unit', required=False)), ('timeseries_chart_type', wagtail.blocks.ChoiceBlock(choices=[('lines', 'Line Chart'), ('bars', 'Bar Chart')], label='Chart Type')), ('timeseries_chart_color', wagtail_color_panel.blocks.NativeColorBlock(default='#367DA4', label='Chart Line/Bar Color', required=True))], label='Point Analysis')), ('area_analysis', wagtail.blocks.StructBlock([('instance_data_enabled', wagtail.blocks.BooleanBlock(default=True, label='Show data for area', required=False)), ('instance_value_type', wagtail.blocks.ChoiceBlock(choices=[('mean', 'Mean of pixel values'), ('sum', 'Sum of pixel values'), ('minmax', 'Minimum, Maximum pixel values'), ('minmeanmax', 'Minimum, Mean, Maximum pixel values')], help_text='The value type that should be displayed', label='Area value type')), ('unit', wagtail.blocks.CharBlock(label='Data unit', required=False)), ('timeseries_data_enabled', wagtail.blocks.BooleanBlock(default=True, label='Show timeseries data for area', required=False)), ('timeseries_aggregation_method', wagtail.blocks.ChoiceBlock(choices=[('mean', 'By Mean'), ('sum', 'By Sum')], help_text='How should the region data be aggregated ?', label='Area timeseries data aggregation Method')), ('timeseries_chart_type', wagtail.blocks.ChoiceBlock(choices=[('lines', 'Line Chart'), ('bars', 'Bar Chart')], label='Chart Type')), ('timeseries_chart_color', wagtail_color_panel.blocks.NativeColorBlock(default='#367DA4', label='circle stroke color', required=True))], label='Area Analysis'))], blank=True, null=True, use_json_field=True, verbose_name='Analysis')),
            ],
            options={
                'get_latest_by': 'modified',
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='RasterTileLayer',
            fields=[
                ('created', django_extensions.db.fields.CreationDateTimeField(auto_now_add=True, verbose_name='created')),
                ('modified', django_extensions.db.fields.ModificationDateTimeField(auto_now=True, verbose_name='modified')),
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('title', models.CharField(help_text='layer title', max_length=255, verbose_name='title')),
                ('default', models.BooleanField(default=False, help_text='Is Default Layer', verbose_name='default')),
                ('base_url', models.CharField(max_length=500, verbose_name='Base raster tile url')),
                ('query_params_static', wagtail.fields.StreamField([('param', wagtail.blocks.StructBlock([('key', wagtail.blocks.CharBlock(label='Key')), ('value', wagtail.blocks.CharBlock(label='Value'))], label='Query Parameter'))], blank=True, help_text='Static query params to be added to the url', null=True, use_json_field=True, verbose_name='Static Query Params')),
                ('query_params_selectable', wagtail.fields.StreamField([('param', wagtail.blocks.StructBlock([('name', wagtail.blocks.CharBlock(label='name')), ('label', wagtail.blocks.CharBlock(label='label', required=False)), ('type', wagtail.blocks.ChoiceBlock(choices=[('radio', 'Radio'), ('dropdown', 'Dropdown')], label='Selector Type')), ('options', wagtail.blocks.ListBlock(wagtail.blocks.StructBlock([('label', wagtail.blocks.CharBlock(label='label')), ('value', wagtail.blocks.CharBlock(label='value')), ('default', wagtail.blocks.BooleanBlock(help_text='Check to make default option', label='default', required=False))]), label='Options', min_num=1))], label='Selectable Query Parameter'))], blank=True, help_text='This should provide a list of options that users can choose to change the query parameter of the url', null=True, use_json_field=True, verbose_name='Query Params With selectable Options')),
                ('params_selectors_side_by_side', models.BooleanField(default=False, verbose_name='Arrange Param Selectors side by side')),
                ('legend', wagtail.fields.StreamField([('legend', wagtail.blocks.StructBlock([('type', wagtail.blocks.ChoiceBlock(choices=[('basic', 'Basic'), ('gradient', 'Gradient'), ('choropleth', 'Choropleth')], label='Legend Type')), ('items', wagtail.blocks.ListBlock(wagtail.blocks.StructBlock([('value', wagtail.blocks.CharBlock(help_text="Can be a number or text e.g '10' or '10-20' or 'Vegetation'", label='value')), ('color', wagtail.blocks.CharBlock(help_text='Color value e.g rgb(73,73,73) or #494949', label='color'))]), label='Legend Items', min_num=1))], label='Custom Legend')), ('legend_image', wagtail.images.blocks.ImageChooserBlock(label='Custom Image'))], blank=True, null=True, use_json_field=True, verbose_name='Legend')),
                ('more_info', wagtail.fields.StreamField([('more_info', wagtail.blocks.StructBlock([('link_text', wagtail.blocks.CharBlock(label='Link text', required=True)), ('link_url', wagtail.blocks.URLBlock(label='Link Url', required=True)), ('text', wagtail.blocks.TextBlock(label='Short description text', required=False)), ('is_button', wagtail.blocks.BooleanBlock(default=True, label='Format link as action button', required=False)), ('show_arrow', wagtail.blocks.BooleanBlock(default=True, label='Show arrow in button', required=False))], label='Info link'))], blank=True, null=True, use_json_field=True, verbose_name='More Info')),
            ],
            options={
                'get_latest_by': 'modified',
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='VectorFileLayer',
            fields=[
                ('created', django_extensions.db.fields.CreationDateTimeField(auto_now_add=True, verbose_name='created')),
                ('modified', django_extensions.db.fields.ModificationDateTimeField(auto_now=True, verbose_name='modified')),
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('title', models.CharField(help_text='layer title', max_length=255, verbose_name='title')),
                ('default', models.BooleanField(default=False, help_text='Is Default Layer', verbose_name='default')),
                ('render_layers', wagtail.fields.StreamField([('fill', wagtail.blocks.StructBlock([('paint', wagtail.blocks.StructBlock([('fill_color', wagtail_color_panel.blocks.NativeColorBlock(default='#000000', label='fill color', required=False)), ('fill_opacity', wagtail.blocks.FloatBlock(default=1.0, label='fill opacity', required=False, validators=[django.core.validators.MinValueValidator(0), django.core.validators.MaxValueValidator(1)])), ('fill_outline_color', wagtail_color_panel.blocks.NativeColorBlock(default='#000000', label='fill outline color', required=False)), ('fill_antialias', wagtail.blocks.BooleanBlock(default=True, label='fill antialias', required=False))], label='Paint Properties')), ('filter', wagtail.blocks.CharBlock(label='filter', required=False)), ('maxzoom', wagtail.blocks.IntegerBlock(label='maxzoom', required=False)), ('minzoom', wagtail.blocks.IntegerBlock(label='minzoom', required=False))], label='Polygon Layer')), ('line', wagtail.blocks.StructBlock([('paint', wagtail.blocks.StructBlock([('line_color', wagtail_color_panel.blocks.NativeColorBlock(default='#000000', label='Line color', required=False)), ('line_dasharray', wagtail.blocks.CharBlock(label='Line dasharray', required=False)), ('line_gap_width', wagtail.blocks.FloatBlock(default=0.0, label='Line gap width', required=False, validators=[django.core.validators.MinValueValidator(0)])), ('line_opacity', wagtail.blocks.FloatBlock(default=1.0, label='line opacity', required=False, validators=[django.core.validators.MinValueValidator(0), django.core.validators.MaxValueValidator(1)])), ('line_width', wagtail.blocks.FloatBlock(default=1.0, label='Line width', required=False, validators=[django.core.validators.MinValueValidator(0)])), ('line_offset', wagtail.blocks.FloatBlock(default=0, label='Line offset', required=False, validators=[django.core.validators.MinValueValidator(0)]))], label='Paint Properties')), ('layout', wagtail.blocks.StructBlock([('line_cap', wagtail.blocks.ChoiceBlock(choices=[('butt', 'Butt'), ('round', 'Round'), ('square', 'Square')], label='Line cap', required=False)), ('line_join', wagtail.blocks.ChoiceBlock(choices=[('miter', 'Miter'), ('bevel', 'Bevel'), ('round', 'Round')], label='Line join', required=False)), ('line_miter_limit', wagtail.blocks.FloatBlock(default=2.0, label='line miter limit', required=False, validators=[django.core.validators.MinValueValidator(0)])), ('line_round_limit', wagtail.blocks.FloatBlock(default=1.05, label='line round limit', required=False, validators=[django.core.validators.MinValueValidator(0)]))], label='Layout Properties')), ('filter', wagtail.blocks.CharBlock(label='filter', required=False)), ('maxzoom', wagtail.blocks.IntegerBlock(label='maxzoom', required=False)), ('minzoom', wagtail.blocks.IntegerBlock(label='minzoom', required=False))], label='Line Layer')), ('circle', wagtail.blocks.StructBlock([('paint', wagtail.blocks.StructBlock([('circle_color', wagtail_color_panel.blocks.NativeColorBlock(default='#000000', label='circle color', required=False)), ('circle_opacity', wagtail.blocks.FloatBlock(default=1.0, label='circle opacity', required=False, validators=[django.core.validators.MinValueValidator(0), django.core.validators.MaxValueValidator(1)])), ('circle_radius', wagtail.blocks.FloatBlock(default=5.0, label='circle radius', required=False, validators=[django.core.validators.MinValueValidator(0)])), ('circle_stroke_color', wagtail_color_panel.blocks.NativeColorBlock(default='#000000', label='circle stroke color', required=False)), ('circle_stroke_width', wagtail.blocks.FloatBlock(default=0.0, label='circle_stroke_width', required=False, validators=[django.core.validators.MinValueValidator(0)]))], label='Paint Properties')), ('filter', wagtail.blocks.CharBlock(label='filter', required=False)), ('maxzoom', wagtail.blocks.IntegerBlock(label='maxzoom', required=False)), ('minzoom', wagtail.blocks.IntegerBlock(label='minzoom', required=False))], label='Point Layer')), ('icon', wagtail.blocks.StructBlock([('layout', wagtail.blocks.StructBlock([('icon_image', wagtailiconchooser.blocks.IconChooserBlock(label='Icon Image')), ('icon_allow_overlap', wagtail.blocks.BooleanBlock(default=False, label='Icon allow overlap', required=False)), ('icon_anchor', wagtail.blocks.ChoiceBlock(choices=[('center', 'Center'), ('left', 'Left'), ('right', 'Right'), ('top', 'Top'), ('bottom', 'Bottom'), ('top-left', 'Top Left'), ('top-right', 'Top Right'), ('bottom-left', 'Bottom Left'), ('bottom-right', 'Bottom Right')], label='Icon anchor', required=False)), ('icon_ignore_placement', wagtail.blocks.BooleanBlock(default=False, label='Icon ignore placement', required=False)), ('icon_keep_upright', wagtail.blocks.BooleanBlock(default=False, label='Icon keep upright', required=False)), ('icon_offset', wagtail.blocks.CharBlock(label='Icon offset', required=False)), ('icon_optional', wagtail.blocks.BooleanBlock(default=False, label='Icon optional', required=False)), ('icon_padding', wagtail.blocks.FloatBlock(default=2.0, label='Icon padding', required=False, validators=[django.core.validators.MinValueValidator(0)])), ('icon_pitch_alignment', wagtail.blocks.ChoiceBlock(choices=[('auto', 'Auto'), ('map', 'Map'), ('viewport', 'Viewport')], label='Icon pitch alignment', required=False)), ('icon_rotate', wagtail.blocks.IntegerBlock(default=0, label='icon rotate', required=False, validators=[django.core.validators.MinValueValidator(0), django.core.validators.MaxValueValidator(360)])), ('icon_rotation_alignment', wagtail.blocks.ChoiceBlock(choices=[('auto', 'Auto'), ('map', 'Map'), ('viewport', 'Viewport')], label='Icon rotation alignment', required=False)), ('icon_size', wagtail.blocks.FloatBlock(default=1, label='icon size', required=False, validators=[django.core.validators.MinValueValidator(0), django.core.validators.MaxValueValidator(1)])), ('icon_text_fit', wagtail.blocks.ChoiceBlock(choices=[('none', 'None'), ('width', 'Width'), ('height', 'Height'), ('both', 'Both')], label='Icon text fit', required=False))], label='Layout Properties')), ('paint', wagtail.blocks.StructBlock([('icon_color', wagtail_color_panel.blocks.NativeColorBlock(default='#000000', label='Icon color', required=False)), ('icon_halo_blur', wagtail.blocks.FloatBlock(default=0.0, label='Icon halo blur', required=False, validators=[django.core.validators.MinValueValidator(0)])), ('icon_halo_color', wagtail_color_panel.blocks.NativeColorBlock(default='#000000', label='Icon halo color', required=False)), ('icon_halo_width', wagtail.blocks.FloatBlock(default=0.0, label='icon halo width', required=False, validators=[django.core.validators.MinValueValidator(0)])), ('icon_opacity', wagtail.blocks.FloatBlock(default=1.0, label='icon opacity', required=False, validators=[django.core.validators.MinValueValidator(0), django.core.validators.MaxValueValidator(1)]))], label='Paint Properties'))], label='Icon Layer')), ('text', wagtail.blocks.StructBlock([('paint', wagtail.blocks.StructBlock([('text_color', wagtail_color_panel.blocks.NativeColorBlock(default='#000000', label='Text color', required=False)), ('text_halo_blur', wagtail.blocks.FloatBlock(default=0.0, label='Text halo blur', required=False, validators=[django.core.validators.MinValueValidator(0)])), ('text_halo_color', wagtail_color_panel.blocks.NativeColorBlock(default='#000000', label='Text halo color', required=False)), ('text_halo_width', wagtail.blocks.FloatBlock(default=0.0, label='text halo width', required=False, validators=[django.core.validators.MinValueValidator(0)])), ('text_translate', wagtail.blocks.CharBlock(label='Text translate', required=False)), ('text_translate_anchor', wagtail.blocks.ChoiceBlock(choices=[('map', 'Map'), ('viewport', 'Viewport')], label='Text translate anchor', required=False))], label='Paint Properties')), ('layout', wagtail.blocks.StructBlock([('symbol_placement', wagtail.blocks.ChoiceBlock(choices=[('point', 'Point'), ('line', 'Line'), ('line-center', 'Line Center')], label='Text Placement', required=False)), ('text_allow_overlap', wagtail.blocks.BooleanBlock(default=False, label='Text allow overlap', required=False)), ('text_anchor', wagtail.blocks.ChoiceBlock(choices=[('center', 'Center'), ('left', 'Left'), ('right', 'Right'), ('top', 'Top'), ('bottom', 'Bottom'), ('top-left', 'Top Left'), ('top-right', 'Top Right'), ('bottom-left', 'Bottom Left'), ('bottom-right', 'Bottom Right')], label='Text anchor', required=False)), ('text_field', wagtail.blocks.CharBlock(label='Text field')), ('text_size', wagtail.blocks.IntegerBlock(default=16, label='Text size', required=False, validators=[django.core.validators.MinValueValidator(0)])), ('text_transform', wagtail.blocks.ChoiceBlock(choices=[('none', 'None'), ('uppercase', 'Uppercase'), ('lowercase', 'Lowercase')], label='Text transform', required=False)), ('text_ignore_placement', wagtail.blocks.BooleanBlock(default=False, label='Text ignore placement', required=False)), ('text_justify', wagtail.blocks.ChoiceBlock(choices=[('center', 'Center'), ('left', 'Left'), ('right', 'Right'), ('auto', 'Auto')], label='Text justify', required=False)), ('text_keep_upright', wagtail.blocks.BooleanBlock(default=False, label='Text keep upright', required=False)), ('text_letter_spacing', wagtail.blocks.FloatBlock(default=0.0, label='Text letter spacing', required=False, validators=[django.core.validators.MinValueValidator(0)])), ('text_line_height', wagtail.blocks.FloatBlock(default=1.2, label='Text line height', required=False, validators=[django.core.validators.MinValueValidator(0)])), ('text_max_angle', wagtail.blocks.IntegerBlock(default=45, label='Text max angle', required=False, validators=[django.core.validators.MinValueValidator(0), django.core.validators.MaxValueValidator(360)])), ('text_max_width', wagtail.blocks.IntegerBlock(default=10, label='Text max width', required=False, validators=[django.core.validators.MinValueValidator(0)])), ('text_offset', wagtail.blocks.CharBlock(label='Text offset', required=False)), ('text_opacity', wagtail.blocks.FloatBlock(default=1.0, label='text opacity', required=False, validators=[django.core.validators.MinValueValidator(0), django.core.validators.MaxValueValidator(1)])), ('text_padding', wagtail.blocks.IntegerBlock(default=2, label='Text  padding', required=False, validators=[django.core.validators.MinValueValidator(0)])), ('text_pitch_alignment', wagtail.blocks.ChoiceBlock(choices=[('auto', 'Auto'), ('map', 'Map'), ('viewport', 'Viewport')], label='Text pitch alignment', required=False)), ('text_radial_offset', wagtail.blocks.IntegerBlock(default=0, label='Text radial offset', required=False, validators=[django.core.validators.MinValueValidator(0)])), ('text_rotate', wagtail.blocks.IntegerBlock(default=0, label='Text rotate', required=False, validators=[django.core.validators.MinValueValidator(0), django.core.validators.MaxValueValidator(360)])), ('text_rotation_alignment', wagtail.blocks.ChoiceBlock(choices=[('auto', 'Auto'), ('map', 'Map'), ('viewport', 'Viewport')], label='Text rotation alignment', required=False)), ('text_variable_anchor', wagtail.blocks.ChoiceBlock(choices=[('center', 'Center'), ('left', 'Left'), ('right', 'Right'), ('top', 'Top'), ('bottom', 'Bottom'), ('top-left', 'Top Left'), ('top-right', 'Top Right'), ('bottom-left', 'Bottom Left'), ('bottom-right', 'Bottom Right')], label='Text variable anchor', required=False))], label='Layout Properties'))], label='Text Label Layer'))], blank=True, null=True, use_json_field=True, verbose_name='Render Layers')),
                ('legend', wagtail.fields.StreamField([('legend', wagtail.blocks.StructBlock([('type', wagtail.blocks.ChoiceBlock(choices=[('basic', 'Basic'), ('gradient', 'Gradient'), ('choropleth', 'Choropleth')], label='Legend Type')), ('items', wagtail.blocks.ListBlock(wagtail.blocks.StructBlock([('value', wagtail.blocks.CharBlock(help_text="Can be a number or text e.g '10' or '10-20' or 'Vegetation'", label='value')), ('color', wagtail.blocks.CharBlock(help_text='Color value e.g rgb(73,73,73) or #494949', label='color'))]), label='Legend Items', min_num=1))], label='Legend')), ('legend_image', wagtail.images.blocks.ImageChooserBlock(label='Legend Image')), ('legend_icon', wagtail.blocks.StructBlock([('items', wagtail.blocks.ListBlock(wagtail.blocks.StructBlock([('icon_image', wagtailiconchooser.blocks.IconChooserBlock(label='Icon Image')), ('icon_label', wagtail.blocks.CharBlock(label='Icon Label')), ('icon_color', wagtail_color_panel.blocks.NativeColorBlock(default='#000000', label='Icon color', required=False))]), label='Legend Icons', min_num=1))], label='Legend Icon'))], blank=True, null=True, use_json_field=True, verbose_name='Legend')),
            ],
            options={
                'get_latest_by': 'modified',
                'abstract': False,
            },
        ),
        migrations.RemoveField(
            model_name='fileimagelayer',
            name='dataset',
        ),
        migrations.RemoveField(
            model_name='fileimagelayer',
            name='style',
        ),
        migrations.RemoveField(
            model_name='vectorlayer',
            name='dataset',
        ),
        migrations.AlterField(
            model_name='dataset',
            name='layer_type',
            field=models.CharField(choices=[('raster_file', 'Raster File - NetCDF/GeoTiff'), ('vector_file', 'Vector File - Shapefile, Geojson'), ('wms', 'Web Map Service - WMS Layer'), ('raster_tile', 'XYZ Raster Tile Layer'), ('vector_tile', 'XYZ Vector Tile Layer')], default='raster_file', max_length=100, verbose_name='Layer type'),
        ),
        migrations.AlterField(
            model_name='dataset',
            name='multi_layer',
            field=models.BooleanField(default=False, help_text='The dataset has more than one layer, to be displayed together', verbose_name='Multi-layer'),
        ),
        migrations.AlterField(
            model_name='dataset',
            name='multi_temporal',
            field=models.BooleanField(default=True, help_text='The dataset is multi-temporal', verbose_name='Multi-temporal'),
        ),
        migrations.AlterField(
            model_name='dataset',
            name='near_realtime',
            field=models.BooleanField(default=False, help_text='Is the layer near realtime?, for example updates every 10 minutes', verbose_name='Near realtime'),
        ),
        migrations.AlterField(
            model_name='dataset',
            name='sub_category',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='geomanager.subcategory', verbose_name='Sub category'),
        ),
        migrations.DeleteModel(
            name='TmsLayer',
        ),
        migrations.AddField(
            model_name='vectorfilelayer',
            name='dataset',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='vector_file_layers', to='geomanager.dataset', verbose_name='dataset'),
        ),
        migrations.AddField(
            model_name='rastertilelayer',
            name='dataset',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='raster_tile_layers', to='geomanager.dataset', verbose_name='dataset'),
        ),
        migrations.AddField(
            model_name='rasterfilelayer',
            name='dataset',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='raster_file_layers', to='geomanager.dataset', verbose_name='dataset'),
        ),
        migrations.AddField(
            model_name='rasterfilelayer',
            name='style',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='geomanager.rasterstyle', verbose_name='style'),
        ),
        migrations.AlterField(
            model_name='layerrasterfile',
            name='layer',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='raster_files', to='geomanager.rasterfilelayer', verbose_name='layer'),
        ),
        migrations.AlterField(
            model_name='pgvectortable',
            name='layer',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='vector_tables', to='geomanager.vectorfilelayer'),
        ),
        migrations.AlterField(
            model_name='vectorlayericon',
            name='layer',
            field=modelcluster.fields.ParentalKey(on_delete=django.db.models.deletion.CASCADE, related_name='icons', to='geomanager.vectorfilelayer'),
        ),
        migrations.DeleteModel(
            name='FileImageLayer',
        ),
        migrations.DeleteModel(
            name='VectorLayer',
        ),
    ]



from django.db import models


class ListField(models.CharField):
    description = "List field"

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def from_db_value(self, value, expression, connection):
        if value is None:
            return value
        return value.split(',')

    def to_python(self, value):
        if isinstance(value, list):
            return value
        if value is None:
            return value
        return value.split(',')

    def get_prep_value(self, value):
        if isinstance(value, list):
            return ','.join([str(i) for i in value])
        return value

    def value_to_string(self, obj):
        value = self.value_from_object(obj)
        return self.get_prep_value(value)



from django.conf import settings
from django.db import models
from django.utils.text import gettext_lazy as _
from django_extensions.db.models import TimeStampedModel


class GeoManagerUserProfile(TimeStampedModel):
    user = models.OneToOneField(settings.AUTH_USER_MODEL, on_delete=models.CASCADE, verbose_name=_("User"))

    gender = models.CharField(max_length=50, verbose_name=_("Gender"))
    country = models.CharField(max_length=50, verbose_name=_("Country"))
    city = models.CharField(max_length=255, verbose_name=_("City"))
    sector = models.TextField(verbose_name=_("Sector"))
    organization = models.CharField(max_length=255, verbose_name=_("Organization"))
    organization_type = models.TextField(max_length=255, verbose_name=_("Organization Type"))
    scale_of_operations = models.CharField(max_length=255, verbose_name=_("Scale of Operations"))
    position = models.CharField(max_length=255, verbose_name=_("Position"))
    how_do_you_use = models.CharField(max_length=255, null=True, blank=True, verbose_name=_("How do you Use"))
    interests = models.TextField(null=True, blank=True, verbose_name=_("Interests"))

    class Meta:
        verbose_name = _("GeoManager User Profile")
        verbose_name_plural = _("GeoManager User Profiles")



import json
import os

from django.conf import settings
from django.core.exceptions import ObjectDoesNotExist
from django.db import connection, close_old_connections
from django.http import JsonResponse, HttpResponse
from django.shortcuts import get_object_or_404, render, redirect
from django.template.defaultfilters import filesizeformat
from django.template.loader import render_to_string
from django.urls import reverse
from django.utils.decorators import method_decorator
from django.utils.translation import gettext as _
from django.views import View
from wagtail.admin import messages
from wagtail.admin.auth import user_passes_test, user_has_any_page_permission, permission_denied
from wagtail.models import Site
from wagtail.snippets.permissions import get_permission_name
from wagtail_modeladmin.helpers import AdminURLHelper
from wagtailcache.cache import cache_page, clear_cache

from geomanager.forms import VectorLayerFileForm, VectorTableForm
from geomanager.models import Dataset
from geomanager.models.core import GeomanagerSettings
from geomanager.models.geostore import Geostore
from geomanager.models.vector_file import VectorUpload, PgVectorTable, VectorFileLayer, VectorLayerIcon
from geomanager.serializers.vector_file import VectorFileLayerSerializer
from geomanager.settings import geomanager_settings
from geomanager.utils import UUIDEncoder
from geomanager.utils.vector_utils import ogr_db_import

ALLOWED_VECTOR_EXTENSIONS = ["zip", "geojson", "csv"]


@user_passes_test(user_has_any_page_permission)
def upload_vector_file(request, dataset_id=None, layer_id=None):
    permission = get_permission_name('change', Dataset)
    if not request.user.has_perm(permission):
        return permission_denied(request)

    site = Site.objects.get(is_default_site=True)
    layer_manager_settings = GeomanagerSettings.for_site(site)

    file_error_messages = {
        "invalid_file_extension": _(
            "Not a supported vector format. Supported formats: %(supported_formats)s."
        ) % {"supported_formats": ALLOWED_VECTOR_EXTENSIONS},
        "file_too_large": _(
            "This file is too big (%(file_size)s). Maximum filesize %(max_filesize)s."
        ),
        "file_too_large_unknown_size": _(
            "This file is too big. Maximum filesize %(max_filesize)s."
        ) % {"max_filesize": filesizeformat(layer_manager_settings.max_upload_size_bytes)}}

    layer = None
    context = {}
    context.update(
        {
            "max_filesize": layer_manager_settings.max_upload_size_bytes,
            "allowed_extensions": ALLOWED_VECTOR_EXTENSIONS,
            "error_max_file_size": file_error_messages["file_too_large_unknown_size"],
            "error_accepted_file_types": file_error_messages["invalid_file_extension"],
        }
    )

    dataset = get_object_or_404(Dataset, pk=dataset_id)

    admin_url_helper = AdminURLHelper(Dataset)
    dataset_list_url = admin_url_helper.get_action_url("index")
    layer_list_url = None
    layer_preview_url = None

    context.update({"dataset": dataset, "layer": layer, "datasets_index_url": dataset_list_url,
                    "layers_index_url": layer_list_url, "dataset_preview_url": dataset.preview_url,
                    "layer_preview_url": layer_preview_url})

    # Check if user is submitting
    if request.method == 'POST':
        files = request.FILES.getlist('files[]', None)
        file = files[0]

        upload = VectorUpload.objects.create(file=file, dataset=dataset)
        upload.save()

        query_set = VectorFileLayer.objects.filter(dataset=dataset)

        filename = os.path.splitext(upload.file.name)[0]
        filename_without_ext = os.path.basename(filename)

        initial_data = {
            "layer": layer_id if layer_id else query_set.first(),
            "table_name": filename_without_ext
        }

        form_kwargs = {}

        layer_form = VectorLayerFileForm(queryset=query_set, initial=initial_data, **form_kwargs)

        ctx = {
            "form": layer_form,
            "dataset": dataset,
            "publish_action": reverse("geomanager_publish_vector", args=[upload.pk]),
            "delete_action": reverse("geomanager_delete_vector_upload", args=[upload.pk]),
        }

        response = {
            "success": True,
        }

        form = render_to_string(
            "geomanager/vector_file_edit_form.html",
            ctx,
            request=request,
        )
        response.update({"form": form})

        return JsonResponse(response)

    return render(request, 'geomanager/vector_file_upload.html', context)


@user_passes_test(user_has_any_page_permission)
def publish_vector(request, upload_id):
    if request.method != 'POST':
        return JsonResponse({"message": _("Only POST allowed")})

    upload = VectorUpload.objects.get(pk=upload_id)

    if not upload:
        return JsonResponse({"message": "upload not found"}, status=404)

    db_layer = get_object_or_404(VectorFileLayer, pk=request.POST.get('layer'))

    form_kwargs = {}

    data = {
        "layer": db_layer,
        "time": request.POST.get('time'),
        "table_name": request.POST.get('table_name'),
        "description": request.POST.get('description'),
    }

    queryset = VectorFileLayer.objects.filter(dataset=upload.dataset)
    layer_form = VectorLayerFileForm(data=data, queryset=queryset, **form_kwargs)

    ctx = {
        "dataset": upload.dataset,
        "publish_action": reverse("geomanager_publish_vector", args=[upload.pk]),
        "delete_action": reverse("geomanager_delete_vector_upload", args=[upload.pk]),
        "form": layer_form
    }

    def get_response():
        return {
            "success": False,
            "form": render_to_string(
                "geomanager/vector_file_edit_form.html",
                ctx,
                request=request,
            ),
        }

    if layer_form.is_valid():
        layer = layer_form.cleaned_data['layer']
        time = layer_form.cleaned_data['time']
        table_name = layer_form.cleaned_data['table_name']
        if table_name:
            table_name = table_name.lower()
        description = layer_form.cleaned_data['description']

        exists = PgVectorTable.objects.filter(layer=db_layer, time=time, table_name=table_name).exists()

        if exists:
            error_message = _("File with date %(time)s already exists for selected layer") % {"time": time.isoformat()}
            layer_form.add_error("time", error_message)
            return JsonResponse(get_response())

        data = {
            "layer": layer,
            "time": time,
            "table_name": table_name,
            "description": description
        }

        default_db_settings = settings.DATABASES['default']

        db_params = {
            "host": default_db_settings.get("HOST"),
            "port": default_db_settings.get("PORT"),
            "user": default_db_settings.get("USER"),
            "password": default_db_settings.get("PASSWORD"),
            "name": default_db_settings.get("NAME"),
        }

        db_settings = {
            **db_params,
            "pg_service_schema": geomanager_settings.get("vector_db_schema")
        }

        table_info = ogr_db_import(upload.file.path, table_name, db_settings)
        full_table_name = table_info.get("table_name")
        properties = table_info.get("properties")
        bounds = table_info.get("bounds")
        geom_type = table_info.get("geom_type")

        data.update({
            "full_table_name": full_table_name,
            "properties": properties,
            "bounds": bounds,
            "geometry_type": geom_type
        })

        PgVectorTable.objects.create(**data)

        # cleanup
        upload.delete()
        return JsonResponse({"success": True, })
    else:
        return JsonResponse(get_response())


@user_passes_test(user_has_any_page_permission)
def delete_vector_upload(request, upload_id):
    if request.method != 'POST':
        return JsonResponse({"message": _("Only POST allowed")})

    upload = VectorUpload.objects.filter(pk=upload_id)

    if upload.exists():
        upload.first().delete()
    else:
        return JsonResponse({"success": True})
    return JsonResponse({"success": True, })


@user_passes_test(user_has_any_page_permission)
def preview_vector_layers(request, dataset_id, layer_id=None):
    template_name = 'geomanager/vector_file_layer_preview.html'
    dataset = get_object_or_404(Dataset, pk=dataset_id)

    base_absolute_url = request.scheme + '://' + request.get_host()

    dataset_admin_helper = AdminURLHelper(Dataset)
    dataset_list_url = dataset_admin_helper.get_action_url("index")

    vector_layer_admin_helper = AdminURLHelper(VectorFileLayer)
    vector_layer_list_url = vector_layer_admin_helper.get_action_url("index")

    geojson_url = request.build_absolute_uri(
        reverse("feature_serv", args=("table_name",)).replace("table_name.geojson", ""))

    data_table = PgVectorTable.objects.filter(layer__id=layer_id)

    if data_table.exists():
        data_table = data_table.first()
    else:
        data_table = None

    initial_data = {
        "columns": data_table.properties if data_table else [],
    }

    dataset_layers = VectorFileLayerSerializer(dataset.vector_file_layers, many=True, context={"request": request}).data

    # get icon images for the dataset vector tile layers, if any
    icon_images = []
    layers_id = [layer.get("id") for layer in dataset_layers]
    for icon in VectorLayerIcon.objects.filter(layer__in=layers_id):
        icon_images.append({"name": icon.name, "url": request.build_absolute_uri(icon.file.url)})

    context = {
        "dataset": dataset,
        "dataset_layers": json.dumps(dataset_layers, cls=UUIDEncoder),
        "selected_layer": layer_id,
        "datasets_index_url": dataset_list_url,
        "vector_layer_list_url": vector_layer_list_url,
        "data_vector_api_base_url": request.build_absolute_uri("/api/vector-data"),
        "vector_tiles_url": base_absolute_url + "/api/vector-tiles/{z}/{x}/{y}",
        "geojson_url": geojson_url,
        "data_table": data_table,
        "icon_images": icon_images
    }

    if request.POST:
        form = VectorTableForm(request.POST, initial=initial_data)
        if form.is_valid():
            columns = form.cleaned_data.get("columns")

            if columns and data_table:
                data_table.properties = columns
                data_table.save()
                # clear wagtail cache
                clear_cache()
            messages.success(request, _("Data fields updated successfully"))
            # redirect
            return redirect(reverse("geomanager_preview_vector_layer", args=(dataset_id, layer_id)))
        else:
            context.update({"form": form})
            return render(request, template_name=template_name, context=context)
    else:
        form = VectorTableForm(initial=initial_data)
        context.update({"form": form})

    return render(request, template_name=template_name, context=context)


@method_decorator(cache_page, name='get')
class VectorTileView(View):
    def get(self, request, z, x, y):
        table_name = request.GET.get("table_name")
        geostore_id = request.GET.get("geostore_id")
        geostore = None

        if not table_name:
            return HttpResponse("Missing 'table_name' query parameter", status=400)

        try:
            vector_table = PgVectorTable.objects.get(table_name=table_name)
            # create list of columns to include in the vector tiles
            columns_sql = ", ".join(vector_table.columns) if vector_table.columns else "*"
        except ObjectDoesNotExist:
            return HttpResponse(f"Table matching 'table_name': {table_name} not found", status=404)

        if geostore_id:
            try:
                geostore = Geostore.objects.get(pk=geostore_id)
            except ObjectDoesNotExist:
                return HttpResponse(f"Geostore matching 'id': {geostore_id} not found", status=404)

            sql = f"""WITH
                        bounds AS (
                          SELECT ST_TileEnvelope(%s, %s, %s) AS geom
                        ),
                        clip_feature AS (
                            SELECT ST_GeomFromText(%s, 4326) AS geom
                        ),
                        mvtgeom AS (
                          SELECT ST_AsMVTGeom(ST_Transform(t.geom, 3857), bounds.geom) AS geom,
                            {columns_sql}
                          FROM {vector_table.full_table_name} t, bounds, clip_feature
                          WHERE ST_Intersects(ST_Transform(t.geom, 4326), ST_Transform(bounds.geom, 4326))
                          AND ST_Intersects(t.geom, clip_feature.geom) 
                        )
                        SELECT ST_AsMVT(mvtgeom, 'default') FROM mvtgeom;
                        """
        else:
            sql = f"""WITH
                        bounds AS (
                          SELECT ST_TileEnvelope(%s, %s, %s) AS geom
                        ),
                        mvtgeom AS (
                          SELECT ST_AsMVTGeom(ST_Transform(t.geom, 3857), bounds.geom) AS geom,
                            {columns_sql}
                          FROM {vector_table.full_table_name} t, bounds
                          WHERE ST_Intersects(ST_Transform(t.geom, 4326), ST_Transform(bounds.geom, 4326))
                        )
                        SELECT ST_AsMVT(mvtgeom, 'default') FROM mvtgeom;
                        """

        close_old_connections()
        with connection.cursor() as cursor:
            try:
                if geostore:
                    cursor.execute(sql, (z, x, y, geostore.geom.wkt))
                else:
                    cursor.execute(sql, (z, x, y))

                tile = cursor.fetchone()[0]
                if not tile:
                    return HttpResponse("Tile not found", status=404)
                return HttpResponse(tile, content_type="application/x-protobuf")
            except Exception as e:
                print("Error", e)
                return HttpResponse(f"Error while fetching tile: {e}", status=500)


@method_decorator(cache_page, name='get')
class GeoJSONPgTableView(View):
    def get(self, request, table_name):
        try:
            vector_table = PgVectorTable.objects.get(table_name=table_name)
        except ObjectDoesNotExist:
            return JsonResponse({"message": f"Table with name: '{table_name}' does not exist"}, status=404)

        property_fields = ", ".join(vector_table.columns) if vector_table.columns else "*"

        close_old_connections()

        with connection.cursor() as cursor:
            query = f"""
                SELECT json_build_object(
                    'type', 'FeatureCollection', 
                    'features', json_agg(feature)
                ) FROM (
                    SELECT json_build_object(
                        'type', 'Feature', 
                        'geometry', ST_AsGeoJSON(geom)::json, 
                        'properties', to_jsonb(inputs) - 'geom'
                    ) AS feature 
                    FROM (
                        SELECT {property_fields}, geom
                        FROM {vector_table.full_table_name}
                    ) AS inputs
                ) AS features;
            """

            cursor.execute(query)
            data = cursor.fetchone()[0]

        return JsonResponse(data)



from rest_framework import serializers

from geomanager.models import Category
from geomanager.models.core import SubCategory, Dataset, Metadata
from geomanager.serializers.raster_file import RasterFileLayerSerializer
from geomanager.serializers.raster_tile import RasterTileLayerSerializer
from geomanager.serializers.vector_file import VectorFileLayerSerializer
from geomanager.serializers.vector_tile import VectorTileLayerSerializer
from geomanager.serializers.wms import WmsLayerSerializer


class DatasetSerializer(serializers.ModelSerializer):
    layers = serializers.SerializerMethodField()
    layer = serializers.SerializerMethodField()
    dataset = serializers.SerializerMethodField()
    name = serializers.SerializerMethodField()
    capabilities = serializers.SerializerMethodField()
    isMultiLayer = serializers.SerializerMethodField()
    initialVisible = serializers.SerializerMethodField()

    class Meta:
        model = Dataset
        fields = ["id", "dataset", "name", "capabilities", "summary", "layer", "isMultiLayer", "category",
                  "sub_category", "metadata", "layers", "initialVisible", "public"]

    def get_initialVisible(self, obj):
        return obj.initial_visible

    def get_isMultiLayer(self, obj):
        return obj.multi_layer

    def get_capabilities(self, obj):
        return obj.capabilities

    def get_layers(self, obj):
        request = self.context.get('request')

        if obj.layer_type == "raster_file":
            return RasterFileLayerSerializer(obj.raster_file_layers, many=True, context={"request": request}).data

        if obj.layer_type == "vector_file":
            return VectorFileLayerSerializer(obj.vector_file_layers, many=True, context={"request": request}).data

        if obj.layer_type == "wms":
            return WmsLayerSerializer(obj.wms_layers, many=True, context={"request": request}).data

        if obj.layer_type == "raster_tile":
            return RasterTileLayerSerializer(obj.raster_tile_layers, many=True, context={"request": request}).data

        if obj.layer_type == "vector_tile":
            return VectorTileLayerSerializer(obj.vector_tile_layers, many=True, context={"request": request}).data

        return None

    def get_layer(self, obj):
        return obj.get_default_layer()

    def get_dataset(self, obj):
        return obj.id

    def get_name(self, obj):
        return obj.title


class SubCategorySerializer(serializers.ModelSerializer):
    class Meta:
        model = SubCategory
        fields = "__all__"


class CategorySerializer(serializers.ModelSerializer):
    sub_categories = SubCategorySerializer(many=True, read_only=True)
    icon = serializers.SerializerMethodField()

    class Meta:
        model = Category
        fields = ['id', 'title', 'icon', 'active', 'public', 'sub_categories']

    def get_icon(self, obj):
        return f"icon-{obj.icon}"


class MetadataSerialiazer(serializers.ModelSerializer):
    class Meta:
        model = Metadata
        fields = "__all__"



# Generated by Django 4.1.10 on 2023-07-12 08:04

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('geomanager', '0009_stationsettings_popup_props'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='stationsettings',
            name='popup_props',
        ),
    ]



from django.contrib import admin

from geomanager.models import LayerRasterFile, Geostore


@admin.register(LayerRasterFile)
class LayerRasterFileAdmin(admin.ModelAdmin):
    list_display = ('pk', "time")


@admin.register(Geostore)
class GeostoreAdmin(admin.ModelAdmin):
    list_display = ('pk',)



# Generated by Django 4.1.10 on 2023-07-10 19:32

from django.db import migrations, models
import geomanager.fields


class Migration(migrations.Migration):

    dependencies = [
        ('geomanager', '0006_stationsettings'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='stationsettings',
            name='id_field',
        ),
        migrations.AddField(
            model_name='stationsettings',
            name='bounds',
            field=geomanager.fields.ListField(blank=True, max_length=256, null=True),
        ),
        migrations.AddField(
            model_name='stationsettings',
            name='geom_type',
            field=models.CharField(blank=True, max_length=100, null=True),
        ),
    ]
